@@ -32,6 +32,12 @@ Kcom <- local({
   if(missing(conditional) || is.null(conditional))
     conditional <- !is.poisson(fit)
 
+  restrict <- isTRUE(restrict)
+  if(restrict && !conditional) {
+    warning("restrict=TRUE ignored because conditional=FALSE", call.=FALSE)
+    restrict <- FALSE
+  }
+
 #  rfixed <- !is.null(r) || !is.null(breaks)
   
   # Extract data and window
