@@ -1,7 +1,7 @@
 #
 #	Kest.R		Estimation of K function
 #
-#	$Revision: 5.120 $	$Date: 2017/06/05 10:31:58 $
+#	$Revision: 5.129 $	$Date: 2020/02/10 09:44:52 $
 #
 #
 # -------- functions ----------------------------------------
@@ -34,8 +34,9 @@
 #
 # ------------------------------------------------------------------------
 
-"Lest" <- function(X, ...) {
-  K <- Kest(X, ...)
+"Lest" <- function(X, ..., correction) {
+  if(missing(correction)) correction <- NULL
+  K <- Kest(X, ..., correction=correction)
   L <- eval.fv(sqrt(K/pi), dotonly=FALSE)
   # handle variance estimates
   if(any(varcols <- colnames(K) %in% c("rip", "ls"))) {
@@ -727,28 +728,29 @@ Knone.engine <- function(X, rmax, nr=100,
      
 
 rmax.rule <- function(fun="K", W, lambda) {
-  verifyclass(W, "owin")
+  if(gotW <- !missing(W)) verifyclass(W, "owin")
+  if(gotL <- !missing(lambda)) lambda <- as.numeric(lambda) # can be vector
+  gotall <- gotW && gotL
   switch(fun,
          K = {
-           # Ripley's Rule
-           ripley <- min(diff(W$xrange), diff(W$yrange))/4
-           # Count at most 1000 neighbours per point
-           rlarge <- if(!missing(lambda)) sqrt(1000 /(pi * lambda)) else Inf
+           ## Ripley's Rule
+           ripley <- if(gotW) shortside(Frame(W))/4 else Inf
+           ## Count at most 1000 neighbours per point
+           rlarge <- if(gotL) sqrt(1000 /(pi * lambda)) else Inf
            rmax <- min(rlarge, ripley)
          },
          Kscaled = {
            ## rule of thumb for Kscaled
-           rdiam  <- diameter(as.rectangle(W))/2 * sqrt(lambda)
+           rdiam  <- if(gotall) diameter(Frame(W))/2 * sqrt(lambda) else Inf
            rmax <- min(10, rdiam)
          },
          F = ,
          G = ,
          J = {
            # rule of thumb
-           rdiam  <- diameter(as.rectangle(W))/2
+           rdiam  <- if(gotW) diameter(Frame(W))/2 else Inf
            # Poisson process has F(rlarge) = 1 - 10^(-5)
-           rlarge <-
-             if(!missing(lambda)) sqrt(log(1e5)/(pi * lambda)) else Inf
+           rlarge <- if(gotL) sqrt(log(1e5)/(pi * lambda)) else Inf
            rmax <- min(rlarge, rdiam)
          },
          stop(paste("Unrecognised function type", sQuote(fun)))
@@ -769,7 +771,7 @@ implemented.for.K <- function(correction, windowtype, explicit) {
       if(any(iso)) {
         whinge <- "Isotropic correction not implemented for binary masks"
         if(explicit) {
-          if(all(iso)) stop(whinge) else warning(whinge)
+        if(all(iso)) stop(whinge, call.=FALSE) else warning(whinge, call.=FALSE)
         }
         correction <- correction[!iso]
       }
@@ -790,8 +792,9 @@ good.correction.K <- function(X) {
 }
 
 Krect.engine <- function(X, rmax, nr=100,
-                           correction,
-                           weights=NULL, ratio=FALSE, fname="K") {
+                         correction,
+                         weights=NULL, ratio=FALSE, fname="K",
+                         use.integers=TRUE) {
   verifyclass(X, "ppp")
   npts <- npoints(X)
   W <- as.owin(X)
@@ -883,7 +886,7 @@ Krect.engine <- function(X, rmax, nr=100,
               bdenom=as.double(zbdenom),
               unco=as.double(zunco),
               PACKAGE = "spatstat")
-  } else if(npts < sqrt(.Machine$integer.max)) {
+  } else if(use.integers && npts < sqrt(.Machine$integer.max)) {
     ## unweighted
     ## numerator of border correction can be stored as an integer
     ## use faster integer arithmetic
