@@ -1,7 +1,7 @@
 #
 #	Kinhom.S	Estimation of K function for inhomogeneous patterns
 #
-#	$Revision: 1.93 $	$Date: 2017/07/18 10:14:42 $
+#	$Revision: 1.99 $	$Date: 2019/10/16 03:26:26 $
 #
 #	Kinhom()	compute estimate of K_inhom
 #
@@ -44,8 +44,9 @@
 #
 # ------------------------------------------------------------------------
 
-"Linhom" <- function(...) {
-  K <- Kinhom(...)
+"Linhom" <- function(X, ..., correction) {
+  if(missing(correction)) correction <- NULL
+  K <- Kinhom(X, ..., correction=correction)
   L <- eval.fv(sqrt(pmax.int(K,0)/pi))
   # relabel the fv object
   L <- rebadge.fv(L, quote(L[inhom](r)), c("L", "inhom"),
@@ -88,6 +89,9 @@
 
     # match corrections
     correction.given <- !missing(correction) && !is.null(correction)
+    if(is.null(correction))
+      correction <- c("border", "bord.modif", "isotropic", "translate")
+
     correction <- pickoption("correction", correction,
                              c(none="none",
                                border="border",
@@ -174,12 +178,9 @@
             if(is.ppm(model)) {
               model <- update(model, Q=X)
               lambda <- fitted(model, dataonly=TRUE, leaveoneout=leaveoneout)
-            } else if(is.kppm(model)) {
-              model <- update(model, X=X)
-              lambda <- fitted(model, dataonly=TRUE, leaveoneout=leaveoneout)
             } else {
               model <- update(model, X=X)
-              lambda <- fitted(model, dataonly=TRUE)
+              lambda <- fitted(model, dataonly=TRUE, leaveoneout=leaveoneout)
             }
             danger <- FALSE
             if(miss.update) 
@@ -418,7 +419,7 @@
 }
 
 
-Kwtsum <- function(dIJ, bI, wIJ, b, w, breaks) {
+Kwtsum <- function(dIJ, bI, wIJ, b, w, breaks, fatal=TRUE) {
   #
   # "internal" routine to compute border-correction estimates of Kinhom
   #
@@ -436,11 +437,31 @@ Kwtsum <- function(dIJ, bI, wIJ, b, w, breaks) {
   stopifnot(length(bI) == length(wIJ))
   stopifnot(length(w) == length(b))
 
-  if(!is.finite(sum(w, wIJ)))
-    stop("Weights in K-function were infinite or NA")
-
-  bkval <- breaks$val
+  if(!is.finite(sum(w, wIJ))) {
+    if(fatal)
+      stop("Weights in K-function were infinite or NA", call.=FALSE)
+    #' set non-finite weights to zero
+    if(any(bad <- !is.finite(w))) {
+      warning(paste(sum(bad), "out of", length(bad),
+                    paren(percentage(bad)), 
+                    "of the boundary weights",
+                    "in the K-function were NA or NaN or Inf",
+                    "and were reset to zero"),
+              call.=FALSE)
+      w[bad] <- 0
+    }
+    if(any(bad <- !is.finite(wIJ))) {
+      warning(paste(sum(bad), "out of", length(bad),
+                    paren(percentage(bad)),
+                    "of the weights for pairwise distances",
+                    "in the K-function were NA or NaN or Inf",
+                    "and were reset to zero"),
+              call.=FALSE)
+      wIJ[bad] <- 0
+    }
+  }
   
+  bkval <- breaks$val
   # determine which distances d_{ij} were observed without censoring
   uncen <- (dIJ <= bI)
   #
@@ -483,3 +504,40 @@ validate.weights <- function(x, recip=FALSE, how = NULL,
   }
   return(TRUE)
 }
+
+resolve.lambda <- function(X, lambda=NULL, ...,
+                           sigma=NULL, varcov=varcov,
+                           leaveoneout=TRUE, update=TRUE) {
+  dangerous <- "lambda"
+  danger <- TRUE
+  if(is.null(lambda)) {
+    ## No intensity data provided
+    ## Estimate density by leave-one-out kernel smoothing
+    lambda <- density(X, ..., sigma=sigma, varcov=varcov,
+                      at="points", leaveoneout=leaveoneout)
+    lambda <- as.numeric(lambda)
+    danger <- FALSE
+  } else if(is.im(lambda)) {
+    lambda <- safelookup(lambda, X)
+  } else if(is.function(lambda)) {
+    lambda <- lambda(X$x, X$y)
+  } else if(is.numeric(lambda) && is.vector(as.numeric(lambda))) {
+    check.nvector(lambda, npoints(X))
+  } else if(is.ppm(lambda) || is.kppm(lambda) || is.dppm(lambda)) {
+    model <- lambda
+    if(!update) {
+      ## use intensity of model
+      lambda <- predict(model, locations=X, type="trend")
+    } else {
+      ## re-fit model to data X
+      model <- if(is.ppm(model)) update(model, Q=X) else update(model, X=X)
+      lambda <- fitted(model, dataonly=TRUE, leaveoneout=leaveoneout)
+      danger <- FALSE
+    }
+  } else stop(paste(sQuote("lambda"),
+                    "should be a vector, a pixel image,",
+                    "a fitted model, or a function"))
+  return(list(lambda=lambda,
+              danger=danger,
+              dangerous=if(danger) dangerous else NULL))
+}
