@@ -4,7 +4,7 @@
 #	Compute estimates of cross-type K functions
 #	for multitype point patterns
 #
-#	$Revision: 5.44 $	$Date: 2014/02/15 11:27:34 $
+#	$Revision: 5.50 $	$Date: 2019/10/16 03:09:01 $
 #
 #
 # -------- functions ----------------------------------------
@@ -42,12 +42,13 @@
 #
 # ------------------------------------------------------------------------
 
-"Lcross" <- function(X, i, j, ...) {
+"Lcross" <- function(X, i, j, ..., from, to, correction) {
   if(!is.multitype(X, dfok=FALSE)) 
 	stop("Point pattern must be multitype")
-  if(missing(i)) i <- levels(marks(X))[1]
-  if(missing(j)) j <- levels(marks(X))[2]
-  K <- Kcross(X, i, j, ...)
+  if(missing(i)) i <- if(!missing(from)) from else levels(marks(X))[1]
+  if(missing(j)) j <- if(!missing(to)) to else levels(marks(X))[2]
+  if(missing(correction)) correction <- NULL
+  K <- Kcross(X, i, j, ..., correction=correction)
   L <- eval.fv(sqrt(K/pi))
   # relabel the fv object
   iname <- make.parseable(paste(i))
@@ -62,11 +63,12 @@
   return(L)  
 }
 
-"Ldot" <- function(X, i, ...) {
+"Ldot" <- function(X, i, ..., from, correction) {
   if(!is.multitype(X, dfok=FALSE)) 
 	stop("Point pattern must be multitype")
-  if(missing(i)) i <- levels(marks(X))[1]
-  K <- Kdot(X, i, ...)
+  if(missing(i)) i <- if(!missing(from)) from else levels(marks(X))[1]
+  if(missing(correction)) correction <- NULL
+  K <- Kdot(X, i, ..., correction=correction)
   L <- eval.fv(sqrt(K/pi))
   # relabel the fv object
   iname <- make.parseable(paste(i))
@@ -81,7 +83,7 @@
 "Kcross" <- 
 function(X, i, j, r=NULL, breaks=NULL,
          correction =c("border", "isotropic", "Ripley", "translate") , ...,
-         ratio=FALSE)
+         ratio=FALSE, from, to)
 {
   verifyclass(X, "ppp")
   if(!is.multitype(X, dfok=FALSE)) 
@@ -90,24 +92,28 @@ function(X, i, j, r=NULL, breaks=NULL,
     correction <- NULL
   marx <- marks(X)
   if(missing(i))
-    i <- levels(marx)[1]
+    i <- if(!missing(from)) from else levels(marx)[1]
   if(missing(j))
-    j <- levels(marx)[2]
+    j <- if(!missing(to)) to else levels(marx)[2]
   I <- (marx == i)
   if(!any(I))
     stop(paste("No points have mark i =", i))
 
   if(i == j) {
-    result <- Kest(X[I],
-                   r=r, breaks=breaks, correction=correction, ...,
-                   ratio=ratio)
+    ## use Kest
+    result <- do.call(Kest,
+                      resolve.defaults(list(X=X[I],
+                                            r=r, breaks=breaks,
+                                            correction=correction, ratio=ratio),
+                                       list(rmax=NULL), ## forbidden 
+                                       list(...)))
   } else {
     J <- (marx == j)
     if(!any(J))
       stop(paste("No points have mark j =", j))
     result <- Kmulti(X, I, J,
-                     r=r, breaks=breaks, correction=correction, ...,
-                     ratio=ratio)
+                     r=r, breaks=breaks,
+                     correction=correction, ratio=ratio, ...)
   }
   iname <- make.parseable(paste(i))
   jname <- make.parseable(paste(j))
@@ -123,7 +129,7 @@ function(X, i, j, r=NULL, breaks=NULL,
 "Kdot" <- 
 function(X, i, r=NULL, breaks=NULL,
          correction = c("border", "isotropic", "Ripley", "translate") , ...,
-         ratio=FALSE)
+         ratio=FALSE, from)
 {
   verifyclass(X, "ppp")
   if(!is.multitype(X, dfok=FALSE)) 
@@ -133,7 +139,7 @@ function(X, i, r=NULL, breaks=NULL,
 
   marx <- marks(X)
   if(missing(i))
-    i <- levels(marx)[1]
+    i <- if(!missing(from)) from else levels(marx)[1]
         
   I <- (marx == i)
   J <- rep.int(TRUE, X$n)  # i.e. all points
@@ -160,10 +166,8 @@ function(X, I, J, r=NULL, breaks=NULL,
   verifyclass(X, "ppp")
 
   npts <- npoints(X)
-  x <- X$x
-  y <- X$y
   W <- X$window
-  area <- area.owin(W)
+  areaW <- area(W)
 
   correction.given <- !missing(correction) && !is.null(correction)
   if(is.null(correction))
@@ -193,8 +197,8 @@ function(X, I, J, r=NULL, breaks=NULL,
 		
   nI <- sum(I)
   nJ <- sum(J)
-  lambdaI <- nI/area
-  lambdaJ <- nJ/area
+  lambdaI <- nI/areaW
+  lambdaJ <- nJ/areaW
 
   # r values 
   rmaxdefault <- rmax.rule("K", W, lambdaJ)
@@ -216,7 +220,7 @@ function(X, I, J, r=NULL, breaks=NULL,
   
   # save numerator and denominator?
   if(ratio) {
-    denom <- lambdaI * lambdaJ * area
+    denom <- lambdaI * lambdaJ * areaW
     numK <- eval.fv(denom * K)
     denK <- eval.fv(denom + K * 0)
     attributes(numK) <- attributes(denK) <- attributes(K)
@@ -227,7 +231,7 @@ function(X, I, J, r=NULL, breaks=NULL,
   # find close pairs of points
   XI <- X[I]
   XJ <- X[J]
-  close <- crosspairs(XI, XJ, max(r))
+  close <- crosspairs(XI, XJ, max(r), what="ijd")
 # close$i and close$j are serial numbers in XI and XJ respectively;        
 # map them to original serial numbers in X
   orig <- seq_len(npts)
@@ -241,12 +245,6 @@ function(X, I, J, r=NULL, breaks=NULL,
     if(!all(ok)) {
       close$i  <- close$i[ok]
       close$j  <- close$j[ok]
-      close$xi <- close$xi[ok]
-      close$yi <- close$yi[ok]
-      close$xj <- close$xj[ok]
-      close$yj <- close$yj[ok]
-      close$dx <- close$dx[ok]
-      close$dy <- close$dy[ok]
       close$d  <- close$d[ok]
     }
   }
@@ -261,7 +259,7 @@ function(X, I, J, r=NULL, breaks=NULL,
     # uncorrected! 
     wh <- whist(dcloseIJ, breaks$val)  # no weights
     numKun <- cumsum(wh)
-    denKun <- lambdaI * lambdaJ * area
+    denKun <- lambdaI * lambdaJ * areaW
     Kun <- numKun/denKun
     K <- bind.fv(K, data.frame(un=Kun), "{hat(%s)[%s]^{un}}(r)",
                  "uncorrected estimate of %s",
@@ -329,7 +327,7 @@ function(X, I, J, r=NULL, breaks=NULL,
     edgewt <- edge.Trans(XI[icloseI], XJ[jcloseJ], paired=TRUE)
     wh <- whist(dcloseIJ, breaks$val, edgewt)
     numKtrans <- cumsum(wh)
-    denKtrans <- lambdaI * lambdaJ * area
+    denKtrans <- lambdaI * lambdaJ * areaW
     Ktrans <- numKtrans/denKtrans
     rmax <- diameter(W)/2
     Ktrans[r >= rmax] <- NA
@@ -352,7 +350,7 @@ function(X, I, J, r=NULL, breaks=NULL,
     edgewt <- edge.Ripley(XI[icloseI], matrix(dcloseIJ, ncol=1))
     wh <- whist(dcloseIJ, breaks$val, edgewt)
     numKiso <- cumsum(wh)
-    denKiso <- lambdaI * lambdaJ * area
+    denKiso <- lambdaI * lambdaJ * areaW
     Kiso <- numKiso/denKiso
     rmax <- diameter(W)/2
     Kiso[r >= rmax] <- NA
