@@ -3,11 +3,11 @@
 #
 #   Delta metric
 #
-#   $Revision: 1.3 $  $Date: 2010/07/10 11:42:08 $
+#   $Revision: 1.4 $  $Date: 2014/10/24 00:22:30 $
 #
 
 deltametric <- function(A, B, p=2, c=Inf, ...) {
-  stopifnot(is.numeric(p) && length(p) == 1 && p > 0)
+  stopifnot(is.numeric(p) && length(p) == 1L && p > 0)
   # ensure frames are identical
   bb <- boundingbox(as.rectangle(A), as.rectangle(B))
   # enforce identical frames
