@@ -6,16 +6,21 @@
 #'    Copyright (C) 2017 Greg McSwiggan and Adrian Baddeley
 #'
 
-density.lpp <- function(x, sigma, ...,
+density.lpp <- function(x, sigma=NULL, ...,
                         weights=NULL,
+                        distance=c("path", "euclidean"),
                         kernel="gaussian", 
                         continuous=TRUE,
                         epsilon=1e-6,
                         verbose=TRUE, debug=FALSE, savehistory=TRUE,
                         old=FALSE) {
   stopifnot(inherits(x, "lpp"))
-  kernel <- match.kernel(kernel)
+  distance <- match.arg(distance)
 
+  if(distance == "euclidean") 
+    return(densityQuick.lpp(x, sigma, ..., kernel=kernel, weights=weights))
+  
+  kernel <- match.kernel(kernel)
   if(continuous && (kernel == "gaussian") && !old)
      return(PDEdensityLPP(x, sigma, ..., weights=weights))
 
@@ -57,7 +62,8 @@ density.lpp <- function(x, sigma, ...,
   Llengths <- lengths.psp(Llines)
   # initialise stack
   stack <- data.frame(seg=integer(0), from=logical(0), 
-                  distance=numeric(0), weight=numeric(0))
+                      distance=numeric(0), weight=numeric(0),
+                      generation=integer(0))
   # process each data point
   for(i in seq_len(n)) {
     segi <- seg[i]
@@ -72,7 +78,8 @@ density.lpp <- function(x, sigma, ...,
     stack <- rbind(data.frame(seg = c(segi, segi),
                               from  = c(TRUE, FALSE), 
                               distance = len * c(tpi, 1-tpi),
-                              weight = rep(weights[i], 2L)),
+                              weight = rep(weights[i], 2L),
+                              generation = rep(1L, 2)),
                    stack)
   }
   Lfrom <- L$from
@@ -82,7 +89,12 @@ density.lpp <- function(x, sigma, ...,
   if(savehistory)
     history <- data.frame(iter=integer(0), qlen=integer(0),
                           totmass=numeric(0), maxmass=numeric(0))
-  # process the stack
+
+  lastgen <- resolve.1.default(list(lastgen=Inf), list(...))
+  sortgen <- resolve.1.default(list(sortgen=FALSE), list(...))
+  sortgen <- sortgen || is.finite(lastgen) 
+
+  ## process the stack
   while(nrow(stack) > 0) {
     if(debug) print(stack)
     masses <- with(stack, abs(weight) * pkernel(distance,
@@ -121,6 +133,10 @@ density.lpp <- function(x, sigma, ...,
     Hseg <- H$seg
     Hvert <- if(H$from) Lfrom[Hseg] else Lto[Hseg]
     Hdist <- H$distance
+    Hgen <- H$generation
+    ## finished processing?
+    if(Hgen > lastgen)
+      break;
     # find all segments incident to this vertex
     incident <- which((Lfrom == Hvert) | (Lto == Hvert))
     degree <- length(incident)
@@ -147,8 +163,12 @@ density.lpp <- function(x, sigma, ...,
       stack <- rbind(data.frame(seg = J,
                                 from  = !(H.is.from),
                                 distance = lenJ + Hdist,
-                                weight = Jweight),
+                                weight = Jweight,
+                                generation = Hgen + 1L),
                      stack)
+      if(sortgen)
+        stack <- stack[order(stack$generation), , drop=FALSE]
+      print(stack)
     }
   }
   # attach values to nearest pixels
@@ -162,14 +182,15 @@ density.lpp <- function(x, sigma, ...,
   return(out)
 }
 
-density.splitppx <- function(x, sigma, ...) {
+density.splitppx <- function(x, sigma=NULL, ...) {
   if(!all(sapply(x, is.lpp)))
     stop("Only implemented for patterns on a linear network")
   solapply(x, density.lpp, sigma=sigma, ...)
 }
 
 PDEdensityLPP <- function(x, sigma, ..., weights=NULL, 
-                          dx=NULL, dt=NULL,
+                          dx=NULL, dt=NULL, 
+                          iterMax=1e6,
                           fun=FALSE, 
                           finespacing=FALSE, finedata=finespacing) {
   stopifnot(is.lpp(x))
@@ -196,7 +217,7 @@ PDEdensityLPP <- function(x, sigma, ..., weights=NULL,
              } else if(!is.null(argh$dimyx)) {
                min(sidelengths(W)/argh$dimyx)
              } else if(!is.null(argh$xy)) {
-               with(as.mask(W, xy=xy), min(xstep, ystep))
+               with(as.mask(W, xy=argh$xy), min(xstep, ystep))
              } else min(sidelengths(W)/spatstat.options("npixel"))
       dx <- max(eps/1.4, lbar/30)
     }
@@ -215,7 +236,8 @@ PDEdensityLPP <- function(x, sigma, ..., weights=NULL,
   }
   a <- FDMKERNEL(lppobj=x, sigma=sigma, dtx=dx, dtt=dt,
                  weights=weights,
-                 iterMax=1e6, sparse=TRUE)
+                 iterMax=iterMax, sparse=TRUE,
+                 stepnames=list(time="dt", space="dx"))
   f <- a$kernel_fun
   if(fun) {
     result <- f
@@ -239,7 +261,8 @@ PDEdensityLPP <- function(x, sigma, ..., weights=NULL,
 
 # Greg's code 
 FDMKERNEL <- function(lppobj, sigma, dtt, weights=NULL, iterMax=5000, 
-	              sparse=FALSE, dtx) {
+	              sparse=FALSE, dtx,
+                      stepnames=list(time="dtt", space="dtx")) {
   net2 <- as.linnet(lppobj)
 #  ends1 <- net2$lines$ends
   lenfs <- lengths.psp(as.psp(net2))
@@ -267,9 +290,14 @@ FDMKERNEL <- function(lppobj, sigma, dtt, weights=NULL, iterMax=5000,
     U0[is.na(U0)] <- 0
   } 
   M <- round((sigma^2)/(2*dtt))
-  if(M < 10) stop("No of time iterations must be > 10, decrease dtt")
+  if(M < 10)
+    stop(paste("No of time iterations must be > 10; decrease time step",
+               stepnames[["time"]]))
   if(M > iterMax)
-    stop("No of time iterations exceeds iterMax; increase dtt or increase iterMax")
+    stop(paste0("No of time iterations = ", M,
+                " exceeds maximum number iterMax = ", iterMax, 
+                "; increase time step ", stepnames[["time"]],
+                ", or increase iterMax"))
 
   alpha <- dtt/(dtx^2)
 
@@ -283,7 +311,12 @@ FDMKERNEL <- function(lppobj, sigma, dtt, weights=NULL, iterMax=5000,
   diag(A2) <- 1 - alpha * degree
   
   if(1 - dmax*alpha < 0)
-     stop("alpha must satisfy (1 - HIGHEST VERTEX DEGREE * ALPHA) > 0; decrease dtt or decrease D")
+    stop(paste0("Algorithm is unstable: alpha = ",
+                stepnames[["time"]], "/", stepnames[["space"]], "^2 = ", alpha,
+                " does not satisfy (dmax * alpha <= 1)",
+                " where DMAX = highest vertex degree = ", dmax,
+                "; decrease time step ", stepnames[["time"]],
+                ", or increase spacing ", stepnames[["space"]]))
 
   if(npoints(lppobj) > 0) {
     v <- as.numeric(U0)
