@@ -1,7 +1,7 @@
 ##
 ##    hierhard.R
 ##
-##    $Revision: 1.3 $	$Date: 2017/02/07 07:35:32 $
+##    $Revision: 1.4 $	$Date: 2018/03/15 07:37:41 $
 ##
 ##    The hierarchical hard core process
 ##
@@ -87,7 +87,8 @@ HierHard <- local({
                  "hierarchical order"),
     pardesc  = c("vector of possible types",
                   "matrix of hardcore distances",
-                  "hierarchical order"),
+                 "hierarchical order"),
+    hasInf = TRUE,
     selfstart = function(X, self) {
       types <- self$par$types
       hradii <- self$par$hradii
