@@ -1,7 +1,7 @@
 #
 #       images.R
 #
-#      $Revision: 1.148 $     $Date: 2018/09/28 05:09:24 $
+#      $Revision: 1.157 $     $Date: 2019/08/31 05:19:39 $
 #
 #      The class "im" of raster images
 #
@@ -37,6 +37,12 @@ im <- function(mat, xcol=seq_len(ncol(mat)), yrow=seq_len(nrow(mat)),
   if(!is.null(dim(mat))) {
     nr <- nrow(mat)
     nc <- ncol(mat)
+    if(is.na(nc)) {
+      #' handle one-dimensional tables
+      nc <- 1
+      nr <- length(mat)
+      if(missing(xcol)) xcol <- seq_len(nc)
+    }
     if(length(xcol) != nc)
       stop("Length of xcol does not match ncol(mat)")
     if(length(yrow) != nr)
@@ -125,19 +131,12 @@ levels.im <- function(x) {
 shift.im <- function(X, vec=c(0,0), ..., origin=NULL) {
   verifyclass(X, "im")
   if(!is.null(origin)) {
-    stopifnot(is.character(origin))
     if(!missing(vec))
-      warning("argument vec ignored; overruled by argument origin")
-    origin <- pickoption("origin", origin, c(centroid="centroid",
-                                             midpoint="midpoint",
-                                             bottomleft="bottomleft"))
-    W <- as.owin(X)
-    locn <- switch(origin,
-                   centroid={ unlist(centroid.owin(W)) },
-                   midpoint={ c(mean(W$xrange), mean(W$yrange)) },
-                   bottomleft={ c(W$xrange[1L], W$yrange[1L]) })
-    return(shift(X, -locn))
+      warning("argument vec ignored; argument origin has precedence")
+    locn <- interpretAsOrigin(origin, Window(X))
+    vec <- -locn
   }
+  vec <- as2vector(vec)
   X$xrange <- X$xrange + vec[1L]
   X$yrange <- X$yrange + vec[2L]
   X$xcol <- X$xcol + vec[1L]
@@ -424,7 +423,7 @@ update.im <- function(object, ...) {
   return(X)
 }
 
-"[<-.im" <- function(x, i, j, value) {
+"[<-.im" <- function(x, i, j, ..., drop=TRUE, value) {
   # detect 'blank' arguments like second argument of x[i, ] 
   ngiven <- length(sys.call())
   nmatched <- length(match.call())
@@ -447,14 +446,23 @@ update.im <- function(object, ...) {
     value <- value$v
 
   if(itype == "missing" && jtype == "missing") {
-    # no index provided
-    # set all pixels to 'value'
+    #' no index provided
+    #' set all pixels to 'value'
+    #' (if drop=TRUE, this applies only to pixels inside the window)
     v <- X$v
     if(!is.factor(value)) {
-      v[!is.na(v)] <- value
+      if(!drop) {
+        v[] <- value
+      } else {
+        v[!is.na(v)] <- value
+      }
     } else {
       vnew <- matrix(NA_integer_, ncol(v), nrow(v))
-      vnew[!is.na(v)] <- as.integer(value)
+      if(!drop) {
+        vnew[] <- as.integer(value)
+      } else {
+        vnew[!is.na(v)] <- as.integer(value)
+      }
       v <- factor(vnew, labels=levels(value))
     }
     X$v <- v
@@ -586,10 +594,10 @@ nearest.pixel <- function(x,y, Z) {
   if(length(x) > 0) {
     nr <- Z$dim[1L]
     nc <- Z$dim[2L]
-    cc <- round(1 + (x - Z$xcol[1L])/Z$xstep)
-    rr <- round(1 + (y - Z$yrow[1L])/Z$ystep)
-    cc <- pmax.int(1,pmin.int(cc, nc))
-    rr <- pmax.int(1,pmin.int(rr, nr))
+    cc <- as.integer(round(1 + (x - Z$xcol[1L])/Z$xstep))
+    rr <- as.integer(round(1 + (y - Z$yrow[1L])/Z$ystep))
+    cc <- pmax.int(1L,pmin.int(cc, nc))
+    rr <- pmax.int(1L,pmin.int(rr, nr))
   } else cc <- rr <- integer(0)
   return(list(row=rr, col=cc))
 }
@@ -597,37 +605,77 @@ nearest.pixel <- function(x,y, Z) {
 # Explores the 3 x 3 neighbourhood of nearest.pixel
 # and finds the nearest pixel that is not NA
 
-nearest.valid.pixel <- function(x, y, Z) {
-  rc <- nearest.pixel(x,y,Z) # checks that Z is an 'im' or 'mask'
-  rr <- rc$row
-  cc <- rc$col
-  # check whether any pixels are outside image domain
-  inside <- as.owin(Z)$m
-  miss <- !inside[cbind(rr, cc)]
-  if(!any(miss))
-    return(rc)
-  # for offending pixels, explore 3 x 3 neighbourhood
-  nr <- Z$dim[1L]
-  nc <- Z$dim[2L]
-  xcol <- Z$xcol
-  yrow <- Z$yrow
-  for(i in which(miss)) {
-    rows <- rr[i] + c(-1L,0L,1L)
-    cols <- cc[i] + c(-1L,0L,1L)
-    rows <- unique(pmax.int(1, pmin.int(rows, nr)))
-    cols <- unique(pmax.int(1, pmin.int(cols, nc)))
-    rcp <- expand.grid(row=rows, col=cols)
-    ok <- inside[as.matrix(rcp)]
-    if(any(ok)) {
-      # At least one of the neighbours is valid
-      # Find the closest one
-      rcp <- rcp[ok,]
-      dsq <- with(rcp, (x[i] - xcol[col])^2 + (y[i] - yrow[row])^2)
-      j <- which.min(dsq)
-      rc$row[i] <- rcp$row[j]
-      rc$col[i] <- rcp$col[j]
-    }
-  }
+nearest.valid.pixel <- function(x, y, Z,
+                                method=c("C","interpreted"),
+                                nsearch=1) {
+  method <- match.arg(method)
+  switch(method,
+         interpreted = {
+           rc <- nearest.pixel(x,y,Z) # checks that Z is an 'im' or 'mask'
+           rr <- rc$row
+           cc <- rc$col
+           #' check whether any pixels are outside image domain
+           inside <- as.owin(Z)$m
+           miss <- !inside[cbind(rr, cc)]
+           if(!any(miss))
+             return(rc)
+           #' for offending pixels, explore 3 x 3 neighbourhood
+           nr <- Z$dim[1L]
+           nc <- Z$dim[2L]
+           xcol <- Z$xcol
+           yrow <- Z$yrow
+           searching <- (-nsearch):(nsearch)
+           for(i in which(miss)) {
+             rows <- rr[i] + searching
+             cols <- cc[i] + searching
+             rows <- unique(pmax.int(1L, pmin.int(rows, nr)))
+             cols <- unique(pmax.int(1L, pmin.int(cols, nc)))
+             rcp <- expand.grid(row=rows, col=cols)
+             ok <- inside[as.matrix(rcp)]
+             if(any(ok)) {
+               #' At least one of the neighbours is valid
+               #' Find the closest one
+               rcp <- rcp[ok,]
+               dsq <- with(rcp, (x[i] - xcol[col])^2 + (y[i] - yrow[row])^2)
+               j <- which.min(dsq)
+               rc$row[i] <- rcp$row[j]
+               rc$col[i] <- rcp$col[j]
+             }
+           }
+         },
+         C = {
+           stopifnot(is.im(Z) || is.mask(Z))
+           n <- length(x)
+           if(n == 0) {
+             cc <- rr <- integer(0)
+           } else {
+             nr <- Z$dim[1L]
+             nc <- Z$dim[2L]
+             xscaled <- (x - Z$xcol[1])/Z$xstep
+             yscaled <- (y - Z$yrow[1])/Z$ystep
+             aspect  <- Z$ystep/Z$xstep
+             inside <- as.owin(Z)$m
+             zz <- .C("nearestvalidpixel",
+                      n = as.integer(n),
+                      x = as.double(xscaled),
+                      y = as.double(yscaled),
+                      nr = as.integer(nr),
+                      nc = as.integer(nc),
+                      aspect = as.double(aspect),
+                      z = as.integer(inside),
+                      nsearch = as.integer(nsearch),
+                      rr = as.integer(integer(n)),
+                      cc = as.integer(integer(n)),
+                      PACKAGE="spatstat")
+             rr <- zz$rr + 1L
+             cc <- zz$cc + 1L
+             if(any(bad <- (rr == 0 | cc == 0))) {
+               rr[bad] <- NA
+               cc[bad] <- NA
+             }
+           }
+           rc <- list(row=rr, col=cc)
+         })  
   return(rc)
 }
   
@@ -640,6 +688,11 @@ lookup.im <- function(Z, x, y, naok=FALSE, strict=TRUE) {
 
   if(Z$type == "factor")
     Z <- repair.old.factor.image(Z)
+
+  if((missing(y) || is.null(y)) && all(c("x", "y") %in% names(x))) {
+    y <- x$y
+    x <- x$x
+  }
   
   if(length(x) != length(y))
     stop("x and y must be numeric vectors of equal length")
@@ -1177,3 +1230,19 @@ anyNA.im <- function(x, recursive=FALSE) {
   anyNA(x$v)
 }
 
+ZeroValue <- function(x) {
+  UseMethod("ZeroValue")
+}
+
+ZeroValue.im <- function(x) {
+  lev <- levels(x)
+  z <- switch(x$type,
+              factor  = factor(lev[1L], levels=lev),
+              integer = integer(1L),
+              logical = logical(1L),
+              real    = numeric(1L),
+              complex = complex(1L),
+              character = character(1L),
+              x$v[!is.na(x$v),drop=TRUE][1])
+  return(z)
+}
