@@ -7,5 +7,5 @@ n <- length(r)
 r1 <- c(r[-1L],r[n]+1)
 rm <- (r+r1)/2
 hm <- s(rm)
-identical(all.equal(h,hm),TRUE)
+isTRUE(all.equal(h,hm))
 }
