@@ -3,7 +3,7 @@
 #
 # kluster/kox point process models
 #
-# $Revision: 1.134 $ $Date: 2017/07/19 07:18:54 $
+# $Revision: 1.145 $ $Date: 2020/01/10 05:22:06 $
 #
 
 kppm <- function(X, ...) {
@@ -43,9 +43,9 @@ kppm.formula <-
     thecall[ncall + 1:nargh] <- argh
     names(thecall)[ncall + 1:nargh] <- names(argh)
   }
-#  result <- eval(thecall, 
-#                 envir=if(!is.null(data)) data else parent.frame(),
-#                 enclos=if(!is.null(data)) parent.frame() else baseenv())
+##  result <- eval(thecall, 
+##                 envir=if(!is.null(data)) data else parent.frame(),
+##                 enclos=if(!is.null(data)) parent.frame() else baseenv())
   callenv <- list2env(as.list(data), parent=parent.frame())
   result <- eval(thecall, envir=callenv, enclos=baseenv())
 
@@ -96,7 +96,7 @@ kppm.ppp <- kppm.quad <-
                       rmax = rmax)
   Xenv <- list2env(as.list(covariates), parent=parent.frame())
   X <- eval(substitute(X), envir=Xenv, enclos=baseenv())
-  isquad <- inherits(X, "quad")
+  isquad <- is.quad(X)
   if(!is.ppp(X) && !isquad)
     stop("X should be a point pattern (ppp) or quadrature scheme (quad)")
   if(is.marked(X))
@@ -187,7 +187,8 @@ kppmMinCon <- function(X, Xname, po, clusters, control, statistic, statargs,
               StatName  = fitinfo$StatName,
               FitFun    = fitinfo$FitFun,
               statargs  = statargs,
-              mcfit     = mcfit)
+              mcfit     = mcfit,
+              maxlogcl  = NULL)
   # results
   if(!is.null(DPP)){
     clusters <- update(clusters, as.list(mcfit$par))
@@ -219,17 +220,40 @@ kppmMinCon <- function(X, Xname, po, clusters, control, statistic, statargs,
 }
 
 clusterfit <- function(X, clusters, lambda = NULL, startpar = NULL,
-                       q=1/4, p=2, rmin=NULL, rmax=NULL, ...,
+                       ...,
+                       q=1/4, p=2, rmin=NULL, rmax=NULL, 
+                       ctrl=list(q=q, p=p, rmin=rmin, rmax=rmax),
                        statistic = NULL, statargs = NULL,
-                       algorithm="Nelder-Mead"){
+                       algorithm="Nelder-Mead", verbose=FALSE){
+  if(verbose) splat("Fitting cluster model")
   ## If possible get dataname from dots
   dataname <- list(...)$dataname
   ## Cluster info:
   info <- spatstatClusterModelInfo(clusters)
-  ## Detect DPP usage
+  if(verbose) splat("Retrieved cluster model information")
+  ## Determine model type
+  isPCP <- info$isPCP
   isDPP <- inherits(clusters, "detpointprocfamily")
-  
+
+  ## resolve algorithm parameters
+  default.ctrl <- list(q=if(isDPP) 1/2 else 1/4,
+                       p=2,
+                       rmin=NULL,
+                       rmax=NULL)
+  given.ctrl <- if(missing(ctrl)) list() else ctrl[names(default.ctrl)]
+  given.args <- c(if(missing(q)) NULL else list(q=q),
+                  if(missing(p)) NULL else list(p=p),
+                  if(missing(rmin)) NULL else list(rmin=rmin),
+                  if(missing(rmax)) NULL else list(rmax=rmax))
+  ctrl <- resolve.defaults(given.args, given.ctrl, default.ctrl)
+  if(verbose) {
+    splat("Algorithm parameters:")
+    print(ctrl)
+  }
+  ##
   if(inherits(X, "ppp")){
+      if(verbose) 
+        splat("Using point pattern data")
       if(is.null(dataname))
          dataname <- getdataname(short.deparse(substitute(X), 20), ...)
       if(is.null(statistic))
@@ -238,6 +262,11 @@ clusterfit <- function(X, clusters, lambda = NULL, startpar = NULL,
       if(is.null(startpar))
           startpar <- info$selfstart(X)
       stationary <- is.null(lambda) || (is.numeric(lambda) && length(lambda)==1)
+      if(verbose) {
+        splat("Starting parameters:")
+        print(startpar)
+        cat("Calculating summary function...")
+      }
       # compute summary function
       if(stationary) {
           if(is.null(lambda)) lambda <- intensity(X)
@@ -257,7 +286,10 @@ clusterfit <- function(X, clusters, lambda = NULL, startpar = NULL,
                                            statargs,
                                            list(correction="best")))
       }
+      if(verbose) splat("Done.")
   } else if(inherits(X, "fv")){
+      if(verbose) 
+        splat("Using the given summary function")
       Stat <- X
       ## Get statistic type
       stattype <- attr(Stat, "fname")
@@ -288,9 +320,11 @@ clusterfit <- function(X, clusters, lambda = NULL, startpar = NULL,
   
   ## avoid using g(0) as it may be infinite
   if(statistic=="pcf"){
+      if(verbose) splat("Checking g(0)")
       argu <- fvnames(Stat, ".x")
       rvals <- Stat[[argu]]
       if(rvals[1L] == 0 && (is.null(rmin) || rmin == 0)) {
+          if(verbose) splat("Ignoring g(0)")
           rmin <- rvals[2L]
       }
   }
@@ -298,13 +332,12 @@ clusterfit <- function(X, clusters, lambda = NULL, startpar = NULL,
   ## DPP resolving algorithm and checking startpar
   changealgorithm <- length(startpar)==1 && algorithm=="Nelder-Mead"
   if(isDPP){
+    if(verbose) splat("Invoking dppmFixAlgorithm")
     alg <- dppmFixAlgorithm(algorithm, changealgorithm, clusters, startpar)
     algorithm <- alg$algorithm
   }
 
-  isPCP <- info$isPCP
-  if(isDPP && missing(q)) q <- 1/2
-  dots <- info$resolvedots(..., q = q, p = p, rmin = rmin, rmax = rmax)
+  dots <- info$resolvedots(...)
   # determine initial values of parameters
   startpar <- info$checkpar(startpar)
   # fit
@@ -314,6 +347,7 @@ clusterfit <- function(X, clusters, lambda = NULL, startpar = NULL,
   #' ............ experimental .........................
   do.adjust <- spatstat.options("kppm.adjusted")
   if(do.adjust) {
+    if(verbose) splat("Applying kppm adjustment")
     W <- Window(X)
     adjdata <- list(paircorr = info[["pcf"]],
                     pairWcdf = distcdf(W),
@@ -332,12 +366,14 @@ clusterfit <- function(X, clusters, lambda = NULL, startpar = NULL,
   #' ............ experimental .........................
   usecanonical <- spatstat.options("kppm.canonical")
   if(usecanonical) {
+    if(verbose) splat("Converting to canonical parameters")
      tocanonical <- info$tocanonical
      tohuman <- info$tohuman
      if(is.null(tocanonical) || is.null(tohuman)) {
        warning("Canonical parameters are not yet supported for this model")
        usecanonical <- FALSE
-     }
+     } 
+
   }
   startpar.human <- startpar
   if(usecanonical) {
@@ -345,14 +381,14 @@ clusterfit <- function(X, clusters, lambda = NULL, startpar = NULL,
     startpar <- tocanonical(startpar)
     theoret <- function(par, ...) { htheo(tohuman(par), ...) }
     if(do.adjust)
-      adjustment$data$tohuman <- tohuman
+      adjustment$auxdata$tohuman <- tohuman
   }
   #' ...................................................
   
   mcargs <- resolve.defaults(list(observed=Stat,
                                   theoretical=theoret,
                                   startpar=startpar,
-                                  ctrl=dots$ctrl,
+                                  ctrl=ctrl,
                                   method=algorithm,
                                   fvlab=list(label="%s[fit](r)",
                                       desc=desc),
@@ -364,11 +400,13 @@ clusterfit <- function(X, clusters, lambda = NULL, startpar = NULL,
                                   funaux=info$funaux,
 				  adjustment=adjustment),
                              list(...))
-  if(isDPP && algorithm=="Brent" && changealgorithm){
+
+  if(isDPP && algorithm=="Brent" && changealgorithm)
     mcargs <- resolve.defaults(mcargs, list(lower=alg$lower, upper=alg$upper))
-  }
-  
+
+  if(verbose) splat("Starting minimum contrast fit")
   mcfit <- do.call(mincontrast, mcargs)
+  if(verbose) splat("Returned from minimum contrast fit")
   # extract fitted parameters and reshape
   if(!usecanonical) {
     optpar.canon <- NULL
@@ -390,6 +428,7 @@ clusterfit <- function(X, clusters, lambda = NULL, startpar = NULL,
                   modelname  = info$modelabbrev,
                   lambda     = lambda)
     attr(mcfit, "info") <- extra
+    if(verbose) splat("Returning from clusterfit (DPP case)")
     return(mcfit)
   }
   ## Extra stuff for ordinary cluster/lgcp models
@@ -426,6 +465,7 @@ clusterfit <- function(X, clusters, lambda = NULL, startpar = NULL,
                 isPCP      = isPCP,
                 lambda     = lambda)
   attr(mcfit, "info") <- extra
+  if(verbose) splat("Returning from clusterfit")
   return(mcfit)
 }
 
@@ -541,13 +581,12 @@ kppmComLik <- function(X, Xname, po, clusters, control, weightfun, rmax,
     objargs <- list(dIJ=dIJ, sumweight=sumweight, g=g, gscale=gscale, 
                     envir=environment(paco))
     # define objective function (with 'paco' in its environment)
-    # Note that this is 1/2 of the log composite likelihood,
-    # minus the constant term 
+    # This is the log composite likelihood minus the constant term 
     #       sum(log(lambdaIJ)) - npairs * log(gscale)
     obj <- function(par, objargs) {
       with(objargs,
-           sum(log(paco(dIJ, par)))
-           - sumweight * log(unlist(stieltjes(paco, g, par=par))),
+           2*(sum(log(paco(dIJ, par)))
+             - sumweight * log(unlist(stieltjes(paco, g, par=par)))),
            enclos=objargs$envir)
     }
   } else {
@@ -563,13 +602,12 @@ kppmComLik <- function(X, Xname, po, clusters, control, weightfun, rmax,
     objargs <- list(dIJ=dIJ, wIJ=wIJ, sumweight=sumweight, g=g, gscale=gscale, 
                     envir=environment(wpaco))
     # define objective function (with 'paco', 'wpaco' in its environment)
-    # Note that this is 1/2 of the log composite likelihood,
-    # minus the constant term 
+    # This is the log composite likelihood minus the constant term 
     #       sum(wIJ * log(lambdaIJ)) - sumweight * log(gscale)
     obj <- function(par, objargs) {
       with(objargs,
-           sum(wIJ * log(paco(dIJ, par)))
-           - sumweight * log(unlist(stieltjes(wpaco, g, par=par))),
+           2*(sum(wIJ * log(paco(dIJ, par)))
+             - sumweight * log(unlist(stieltjes(wpaco, g, par=par)))),
            enclos=objargs$envir)
     }
   }
@@ -612,7 +650,8 @@ kppmComLik <- function(X, Xname, po, clusters, control, weightfun, rmax,
                 weightfun = weightfun,
                 rmax      = rmax,
                 objfun    = obj,
-                objargs   = objargs)
+                objargs   = objargs,
+                maxlogcl  = opt$value)
     # pack up
     clusters <- update(clusters, as.list(opt$par))
     result <- list(Xname      = Xname,
@@ -646,7 +685,8 @@ kppmComLik <- function(X, Xname, po, clusters, control, weightfun, rmax,
               weightfun = weightfun,
               rmax      = rmax,
               objfun    = obj,
-              objargs   = objargs)
+              objargs   = objargs,
+              maxlogcl  = opt$value)
   # pack up
   result <- list(Xname      = Xname,
                  X          = X,
@@ -848,7 +888,8 @@ kppmPalmLik <- function(X, Xname, po, clusters, control, weightfun, rmax,
                 weightfun = weightfun,
                 rmax      = rmax,
                 objfun    = obj,
-                objargs   = objargs)
+                objargs   = objargs,
+                maxlogcl  = opt$value)
     # pack up
     clusters <- update(clusters, as.list(optpar.human))
     result <- list(Xname      = Xname,
@@ -880,7 +921,10 @@ kppmPalmLik <- function(X, Xname, po, clusters, control, weightfun, rmax,
   Fit <- list(method    = "palm",
               clfit     = opt,
               weightfun = weightfun,
-              rmax      = rmax)
+              rmax      = rmax,
+              objfun    = obj,
+              objargs   = objargs,
+              maxlogcl  = opt$value)
   # pack up
   result <- list(Xname      = Xname,
                  X          = X,
@@ -1095,8 +1139,13 @@ print.kppm <- print.dppm <- function(x, ...) {
         if(isDPP) "determinantal" else if(isPCP) "cluster" else "Cox",
         "point process model")
 
-  if(waxlyrical('extras', terselevel) && nchar(x$Xname) < 20)
-    splat("Fitted to point pattern dataset", sQuote(x$Xname))
+  Xname <- x$Xname
+  if(waxlyrical('extras', terselevel) && nchar(Xname) < 20) {
+    has.subset <- ("subset" %in% names(x$call))
+    splat("Fitted to",
+          if(has.subset) "(a subset of)" else NULL,
+          "point pattern dataset", sQuote(Xname))
+  }
 
   if(waxlyrical('gory', terselevel)) {
     switch(x$Fit$method,
@@ -1180,6 +1229,11 @@ print.kppm <- print.dppm <- function(x, ...) {
             if(!is.im(mu)) signif(mu, digits) else "[pixel image]")
     }
   }
+  if(isDPP) {
+    rx <- repul(x)
+    splat(if(is.im(rx)) "(Average) strength" else "Strength",
+          "of repulsion:", signif(mean(rx), 4))
+  }
   invisible(NULL)
 }
 
@@ -1338,26 +1392,26 @@ simulate.kppm <- function(object, nsim=1, seed=NULL, ...,
          Thomas={
            kappa <- mp$kappa
            sigma <- mp$sigma
-           cmd <- expression(rThomas(kappa,sigma,mu,win))
+           cmd <- expression(rThomas(kappa,sigma,mu,win, ...))
            dont.complain.about(kappa, sigma, mu)
          },
          MatClust={
            kappa <- mp$kappa
            r     <- mp$R
-           cmd   <- expression(rMatClust(kappa,r,mu,win))
+           cmd   <- expression(rMatClust(kappa,r,mu,win, ...))
            dont.complain.about(kappa, r)
          },
          Cauchy = {
            kappa <- mp$kappa
            omega <- mp$omega
-           cmd   <- expression(rCauchy(kappa, omega, mu, win))
+           cmd   <- expression(rCauchy(kappa, omega, mu, win, ...))
            dont.complain.about(kappa, omega, mu)
          },
          VarGamma = {
            kappa  <- mp$kappa
            omega  <- mp$omega
            nu.ker <- object$covmodel$margs$nu.ker
-           cmd    <- expression(rVarGamma(kappa, nu.ker, omega, mu, win))
+           cmd    <- expression(rVarGamma(kappa, nu.ker, omega, mu, win, ...))
            dont.complain.about(kappa, nu.ker, omega, mu)
          },
          LGCP={
@@ -1432,14 +1486,8 @@ simulate.kppm <- function(object, nsim=1, seed=NULL, ...,
   }
   if(verbose)
     cat("Done.\n")
-  # pack up
-  if(nsim == 1 && drop) {
-    out <- out[[1L]]
-  } else {
-    out <- as.solist(out)
-    if(nsim > 0)
-      names(out) <- paste("Simulation", 1:nsim)
-  }
+  #' pack up
+  out <- simulationresult(out, nsim, drop)
   out <- timed(out, starttime=starttime)
   attr(out, "seed") <- RNGstate
   return(out)
@@ -1504,8 +1552,10 @@ update.kppm <- function(object, ..., evaluate=TRUE) {
   Xexpr <- if(length(jX) > 0) sys.call()[[2L + jX]] else NULL
   #' remove arguments just recognised, if any
   jused <- c(jf, jX)
-  if(length(jused) > 0)
+  if(length(jused) > 0) {
     argh <- argh[-jused]
+    nama <- names(argh)
+  }
   #' update the matched call
   thecall <- getCall(object)
   methodname <- as.character(thecall[[1L]])
@@ -1522,7 +1572,7 @@ update.kppm <- function(object, ..., evaluate=TRUE) {
          },
          {
 	   # original call has X = ppp and trend = [formula without lhs]
-           oldformula <- as.formula(getCall(object)$trend)
+           oldformula <- as.formula(getCall(object)$trend %orifnull% (~1))
 	   fom <-  newformula(oldformula, fmla, callframe, envir)
 	   if(!is.null(Xexpr))
 	      lhs.of.formula(fom) <- Xexpr
@@ -1537,8 +1587,13 @@ update.kppm <- function(object, ..., evaluate=TRUE) {
 	      thecall$X <- fom
 	   }
          })
-  knownnames <- union(names(formals(kppm.ppp)), names(formals(mincontrast)))
-  knownnames <- setdiff(knownnames, c("X", "trend", "observed", "theoretical"))
+  knownnames <- unique(c(names(formals(kppm.ppp)),
+                         names(formals(mincontrast)),
+                         names(formals(optim))))
+  knownnames <- setdiff(knownnames,
+                        c("X", "trend",
+                          "observed", "theoretical",
+                          "fn", "gr", "..."))
   ok <- nama %in% knownnames
   thecall <- replace(thecall, nama[ok], argh[ok])
   thecall$formula <- NULL # artefact of 'step', etc
@@ -1682,30 +1737,27 @@ model.frame.kppm <- model.frame.dppm <- function(formula, ...) {
 }
 
 logLik.kppm <- logLik.dppm <- function(object, ...) {
-  pl <- object$Fit$clfit$value
-  if(is.null(pl))
-    stop("logLik is only available for kppm objects fitted with method='palm'",
+  cl <- object$Fit$maxlogcl
+  if(is.null(cl))
+    stop(paste("logLik is only available for kppm objects fitted with",
+               "method='palm' or method='clik2'"),
          call.=FALSE)
   ll <- logLik(as.ppm(object)) # to inherit class and d.f.
-  ll[] <- pl
+  ll[] <- cl
   return(ll)
 }
  
 AIC.kppm <- AIC.dppm <- function(object, ..., k=2) {
-  # extract Palm loglikelihood
-  pl <- object$Fit$clfit$value
-  if(is.null(pl))
-    stop("AIC is only available for kppm objects fitted with method='palm'",
-         call.=FALSE)
-  df <- length(coef(object))
-  return(- 2 * as.numeric(pl) + k * df)
+  cl <- logLik(object)
+  df <- attr(cl, "df")
+  return(- 2 * as.numeric(cl) + k * df)
 }
 
-extractAIC.kppm <- extractAIC.dppm <- function (fit, scale = 0, k = 2, ...)
-{
-  edf <- length(coef(fit))
-  aic <- AIC(fit, k=k)
-  c(edf, aic)
+extractAIC.kppm <- extractAIC.dppm <- function (fit, scale = 0, k = 2, ...) {
+  cl <- logLik(fit)
+  edf <- attr(cl, "df")
+  aic <- - 2 * as.numeric(cl) + k * edf
+  return(c(edf, aic))
 }
 
 nobs.kppm <- nobs.dppm <- function(object, ...) { nobs(as.ppm(object)) }
