@@ -1,7 +1,7 @@
 #
 # linim.R
 #
-#  $Revision: 1.45 $   $Date: 2017/09/27 09:31:17 $
+#  $Revision: 1.71 $   $Date: 2020/02/05 01:22:28 $
 #
 #  Image/function on a linear network
 #
@@ -15,12 +15,17 @@ linim <- function(L, Z, ..., restrict=TRUE, df=NULL) {
     stopifnot(is.data.frame(df))
     neednames <- c("xc", "yc", "x", "y", "mapXY", "tp", "values")
     ok <- neednames %in% names(df)
-    if(any(!ok)) {
-      nn <- sum(!ok)
-      stop(paste(ngettext(nn, "A column", "Columns"),
-                 "named", commasep(sQuote(neednames[!ok])),
-                 ngettext(nn, "is", "are"),
-                 "missing from argument", sQuote("df")))
+    dfcomplete <- all(ok)
+    if(!dfcomplete) {
+      #' omission of "values" column is permissible, but not other columns
+      mapnames <- setdiff(neednames, "values")
+      if(!all(mapnames %in% names(df))) {
+        nn <- sum(!ok)
+        stop(paste(ngettext(nn, "A column", "Columns"),
+                   "named", commasep(sQuote(neednames[!ok])),
+                   ngettext(nn, "is", "are"),
+                   "missing from argument", sQuote("df")))
+      }
     }
   }
   if(restrict) {
@@ -30,7 +35,7 @@ linim <- function(L, Z, ..., restrict=TRUE, df=NULL) {
       #' ensure all mapped pixels are untouched
       pos <- nearest.pixel(df$xc, df$yc, Z)
       pos <- cbind(pos$row, pos$col)
-      M[pos] <- TRUE
+      M$m[pos] <- TRUE
     }
     Z <- Z[M, drop=FALSE]
   }
@@ -51,6 +56,10 @@ linim <- function(L, Z, ..., restrict=TRUE, df=NULL) {
     values <- Z[pixelcentres]
     # bundle
     df <- cbind(pixdf, projloc, projmap, data.frame(values=values))
+  } else if(!dfcomplete) {
+    #' look up values
+    pixelcentres <- ppp(df$xc, df$yc, window=as.rectangle(Z), check=FALSE)
+    df$values <- safelookup(Z, pixelcentres)
   }
   out <- Z
   attr(out, "L") <- L
@@ -59,6 +68,8 @@ linim <- function(L, Z, ..., restrict=TRUE, df=NULL) {
   return(out)
 }
 
+is.linim <- function(x) { inherits(x, "linim") }
+
 print.linim <- function(x, ...) {
   splat("Image on linear network")
   L <- attr(x, "L")
@@ -133,7 +144,7 @@ print.summary.linim <- function(x, ...) {
 plot.linim <- local({
 
   plot.linim <- function(x, ..., style=c("colour", "width"),
-                         scale, adjust=1,
+                         scale, adjust=1, fatten=0, 
                          negative.args=list(col=2),
                          legend=TRUE,
                          leg.side=c("right", "left", "bottom", "top"),
@@ -142,11 +153,20 @@ plot.linim <- local({
                          leg.args=list(),
                          leg.scale=1,
                          zlim,
+                         box=FALSE,
                          do.plot=TRUE) {
     xname <- short.deparse(substitute(x))
     style <- match.arg(style)
     leg.side <- match.arg(leg.side)
+    check.1.real(leg.scale)
 
+    if(!missing(fatten)) {
+      check.1.real(fatten)
+      if(fatten != 0 && style == "width")
+        warning("Argument 'fatten' is ignored when style='width'", call.=FALSE)
+      stopifnot(fatten >= 0)
+    }
+    
     if(missing(zlim) || is.null(zlim)) {
       zlim <- NULL
       zliminfo <- list()
@@ -156,13 +176,23 @@ plot.linim <- local({
       zliminfo <- list(zlim=zlim)
     }
     
-    ribstuff <- list(ribside  = leg.side,
+    ribstuff <- list(ribbon   = legend,
+                     ribside  = leg.side,
                      ribsep   = leg.sep,
                      ribwid   = leg.wid,
                      ribargs  = leg.args,
                      ribscale = leg.scale)
-    #' colour style: plot as pixel image
-    if(style == "colour" || !do.plot)
+    
+    if(style == "colour" || !do.plot) {
+      #' colour style: plot as pixel image
+      if(fatten > 0) {
+        #' first fatten the lines 
+        L <- attr(x, "L")
+        S <- as.psp(L)
+        D <- distmap(as.mask.psp(S, xy=x))
+        fatwin <- levelset(D, fatten)
+        x <- nearestValue(x)[fatwin, drop=FALSE]
+      }
       return(do.call(plot.im,
                      resolve.defaults(list(x),
                                       list(...),
@@ -170,20 +200,30 @@ plot.linim <- local({
                                       zliminfo, 
                                       list(main=xname,
                                            legend=legend,
-                                           do.plot=do.plot))))
+                                           do.plot=do.plot,
+                                           box=box))))
+    }
     #' width style
     L <- attr(x, "L")
     df <- attr(x, "df")
     Llines <- as.psp(L)
     W <- as.owin(L)
+    #' ensure function values are numeric
+    vals <- try(as.numeric(df$values))
+    if(inherits(vals, "try-error")) 
+      stop("Function values should be numeric: unable to convert them",
+           call.=FALSE)
+    #' convert non-finite values to zero width
+    vals[!is.finite(vals)] <- 0
+    df$values <- vals
     #' plan layout
     if(legend) {
       #' use layout procedure in plot.im
       z <- do.call(plot.im,
-                   resolve.defaults(list(x, do.plot=FALSE, legend=TRUE),
+                   resolve.defaults(list(x, do.plot=FALSE, ribbon=TRUE),
                                     list(...),
                                     ribstuff,
-                                    list(main=xname)))
+                                    list(main=xname, valuesAreColours=FALSE)))
       bb.all <- attr(z, "bbox")
       bb.leg <- attr(z, "bbox.legend")
     } else {
@@ -204,6 +244,8 @@ plot.linim <- local({
                           resolve.defaults(list(x=bb.all, type="n"),
                                            list(...), list(main=xname)),
                           extrargs="type")
+    if(box)
+      plot(Frame(W), add=TRUE)
     #' resolve graphics parameters for polygons
     names(negative.args) <- paste0(names(negative.args), ".neg")
     grafpar <- resolve.defaults(negative.args,
@@ -253,18 +295,18 @@ plot.linim <- local({
       rright <- vv[length(vv)]
       ## first add dodecagonal 'joints'
       if(Ljoined[ileft] && rleft != 0) 
-        pltpoly(x=rleft * dodo$x + leftend$x,
-                y=rleft * dodo$y + leftend$y,
-                grafpar, sign(rleft))
+        drawSignedPoly(x=rleft * dodo$x + leftend$x,
+                      y=rleft * dodo$y + leftend$y,
+                      grafpar, sign(rleft))
       if(Ljoined[iright] && rright != 0)
-        pltpoly(x=rright * dodo$x + rightend$x,
-                y=rright * dodo$y + rightend$y,
-                grafpar, sign(rright))
+        drawSignedPoly(x=rright * dodo$x + rightend$x,
+                      y=rright * dodo$y + rightend$y,
+                      grafpar, sign(rright))
       ## Now render main polygon
       ang <- Lperp[segid]
       switch(signtype,
-             positive = pltseg(xx, yy, vv, ang, grafpar),
-             negative = pltseg(xx, yy, vv, ang, grafpar),
+             positive = drawseg(xx, yy, vv, ang, grafpar),
+             negative = drawseg(xx, yy, vv, ang, grafpar),
              mixed = {
                ## find zero-crossings
                xing <- (diff(sign(vv)) != 0)
@@ -273,57 +315,25 @@ plot.linim <- local({
                elist <- split(data.frame(xx=xx, yy=yy, vv=vv), excu)
                ## plot each excursion
                for(e in elist) 
-                 with(e, pltseg(xx, yy, vv, ang, grafpar))
+                 with(e, drawseg(xx, yy, vv, ang, grafpar))
              })
     }
     result <- adjust * scale
     attr(result, "bbox") <- bb
     if(legend) {
       attr(result, "bbox.legend") <- bb.leg
-      ## get graphical arguments
-      grafpar <- resolve.defaults(leg.args, grafpar)
-      ## set up scale of typical pixel values
-      gvals <- leg.args$at %orifnull% prettyinside(zlim)
-      ## corresponding widths
-      wvals <- adjust * scale * gvals
-      ## glyph positions
-      ng <- length(gvals)
-      xr <- bb.leg$xrange
-      yr <- bb.leg$yrange
-      switch(leg.side,
-             right = ,
-             left = {
-               y <- seq(yr[1], yr[2], length.out=ng+1L)
-               y <- (y[-1L] + y[-(ng+1L)])/2
-               for(j in 1:ng) {
-                 xx <- xr[c(1L,2L,2L,1L)]
-                 yy <- (y[j] + c(-1,1) * wvals[j]/2)[c(1L,1L,2L,2L)]
-                 pltpoly(x = xx, y = yy, grafpar, sign(wvals[j]))
-	     }
-	   },
-	   bottom = ,
-	   top = {
-	     x <- seq(xr[1], xr[2], length.out=ng+1L)
-	     x <- (x[-1L] + x[-(ng+1L)])/2
-	     for(j in 1:ng) {
-	       xx <- (x[j] + c(-1,1) * wvals[j]/2)[c(1L,1L,2L,2L)]
-               yy <- yr[c(1L,2L,2L,1L)]
-               pltpoly(x = xx, y = yy, grafpar, sign(wvals[j]))
-	     }
-	   })
-     # add text labels
-     check.1.real(leg.scale)
-     glabs <- leg.args$labels %orifnull% signif(leg.scale * gvals, 2)
-     switch(leg.side,
-            right  = text(xr[2], y,     pos=4, labels=glabs),
-            left   = text(xr[1], y,     pos=2, labels=glabs),
-	    bottom = text(x,     yr[1], pos=1, labels=glabs),
-	    top    = text(x,     yr[2], pos=3, labels=glabs))
+      plotWidthMap(bb.leg     = bb.leg,
+                   zlim       = zlim,
+                   phys.scale = adjust * scale, 
+                   leg.scale  = leg.scale,
+                   leg.side   = leg.side,
+                   leg.args   = leg.args,
+                   grafpar    = grafpar)
+    }
+    return(invisible(result))
   }
-  return(invisible(result))
-}
 
-  pltseg <- function(xx, yy, vv, ang, pars) {
+  drawseg <- function(xx, yy, vv, ang, pars) {
     ## draw polygon around segment
     sgn <- sign(mean(vv))
     xx <- c(xx, rev(xx))
@@ -331,15 +341,31 @@ plot.linim <- local({
     vv <- c(vv, -rev(vv))
     xx <- xx + cos(ang) * vv
     yy <- yy + sin(ang) * vv
-    pltpoly(xx, yy, pars, sgn)
+    drawSignedPoly(xx, yy, pars, sgn)
     invisible(NULL)
   }
 
+  plot.linim
+})
+
+drawSignedPoly <- local({
+  
+  ## internal function to plot line segments for style="width"
+  ## with sign-dependent colours, etc
+
   pNames <- c("density", "angle", "border", "col", "lty")
   posnames <- paste(pNames, ".pos", sep="")
   negnames <- paste(pNames, ".neg", sep="")
   
-  pltpoly <- function(x, y, pars, sgn) {
+  redub <- function(from, to, x) {
+    #' rename entry x$from to x$to
+    m <- match(from, names(x))
+    if(any(ok <- !is.na(m))) 
+      names(x)[m[ok]] <- to[ok]
+    return(resolve.defaults(x))
+  }
+  
+  drawSignedPoly <- function(x, y, pars, sgn) {
     #' plot polygon using parameters appropriate to "sign"
     if(sgn >= 0) {
       pars <- redub(posnames, pNames, pars)
@@ -351,18 +377,60 @@ plot.linim <- local({
     do.call(polygon, append(list(x=x, y=y), pars))
     invisible(NULL)
   }
-  
-  redub <- function(from, to, x) {
-    #' rename entry x$from to x$to
-    m <- match(from, names(x))
-    if(any(ok <- !is.na(m))) 
-      names(x)[m[ok]] <- to[ok]
-    return(resolve.defaults(x))
-  }
-  
-  plot.linim
+
+  drawSignedPoly
 })
 
+## internal function to plot the map of pixel values to line widths
+
+plotWidthMap <- function(bb.leg, zlim, phys.scale,
+                         leg.scale, leg.side,
+                         leg.args, grafpar) {
+  ## get graphical arguments
+  grafpar <- resolve.defaults(leg.args, grafpar)
+  ## set up scale of typical pixel values
+  gvals <- leg.args$at %orifnull% prettyinside(zlim)
+  ## corresponding widths
+  wvals <- phys.scale * gvals
+  ## glyph positions
+  ng <- length(gvals)
+  xr <- bb.leg$xrange
+  yr <- bb.leg$yrange
+  switch(leg.side,
+         right = ,
+         left = {
+           y <- seq(yr[1], yr[2], length.out=ng+1L)
+           y <- (y[-1L] + y[-(ng+1L)])/2
+           for(j in 1:ng) {
+             xx <- xr[c(1L,2L,2L,1L)]
+             yy <- (y[j] + c(-1,1) * wvals[j]/2)[c(1L,1L,2L,2L)]
+             drawSignedPoly(x = xx, y = yy, grafpar, sign(wvals[j]))
+           }
+         },
+         bottom = ,
+         top = {
+           x <- seq(xr[1], xr[2], length.out=ng+1L)
+           x <- (x[-1L] + x[-(ng+1L)])/2
+           for(j in 1:ng) {
+             xx <- (x[j] + c(-1,1) * wvals[j]/2)[c(1L,1L,2L,2L)]
+             yy <- yr[c(1L,2L,2L,1L)]
+             drawSignedPoly(x = xx, y = yy, grafpar, sign(wvals[j]))
+           }
+         })
+  ## add text labels
+  glabs <- signif(leg.scale * gvals, 2)
+  textpos <- switch(leg.side,
+                    right  = list(x=xr[2], y=y,     pos=4),
+                    left   = list(x=xr[1], y=y,     pos=2),
+                    bottom = list(x=x,     y=yr[1], pos=1),
+                    top    = list(x=x,     y=yr[2], pos=3))
+  textargs <- resolve.defaults(textpos,
+                               leg.args,
+                               list(labels=glabs))
+  do.call.matched(text, textargs, extrargs=graphicsPars("text"))
+  return(invisible(NULL))
+}
+
 sortalongsegment <- function(df) {
   df[fave.order(df$tp), , drop=FALSE]
 }
@@ -439,7 +507,7 @@ as.linim.linim <- function(X, ...) {
 
 # analogue of eval.im
 
-eval.linim <- function(expr, envir, harmonize=TRUE) {
+eval.linim <- function(expr, envir, harmonize=TRUE, warn=TRUE) {
   sc <- sys.call()
   # Get names of all variables in the expression
   e <- as.expression(substitute(expr))
@@ -540,6 +608,10 @@ as.linnet.linim <- function(X, ...) {
   if(!missing(i) && is.lpp(i)) {
     n <- npoints(i)
     result <- vector(mode=typeof(x$v), length=n)
+    if(is.factor(x$v)) {
+      lev <- levels(x$v)
+      result <- factor(result, levels=seq_along(lev), labels=lev)
+    }
     if(n == 0) return(result)
     if(!is.null(df <- attr(x, "df"))) {
       #' use data frame of sample points along network
@@ -569,6 +641,7 @@ as.linnet.linim <- function(X, ...) {
   #' apply subset method for 'im'
   y <- NextMethod("[")
   if(!is.im(y)) return(y) # vector of pixel values
+  class(y) <- unique(c("linim", class(y)))
   #' handle linear network info
   L <- attr(x, "L")
   df <- attr(x, "df")
@@ -612,6 +685,11 @@ as.linnet.linim <- function(X, ...) {
 
 integral.linim <- function(f, domain=NULL, ...){
   verifyclass(f, "linim")
+  if(is.tess(domain)) {
+    result <- sapply(tiles(domain), integral.linim, f = f)
+    if(length(dim(result)) > 1) result <- t(result)
+    return(result)
+  }
   if(!is.null(domain)) 
     f <- f[domain]
   #' extract data
@@ -625,19 +703,27 @@ integral.linim <- function(f, domain=NULL, ...){
   if(any(missed <- (nper == 0))) {
     missed <- unname(which(missed))
     mp <- midpoints.psp(as.psp(L)[missed])
-    mp <- lpp(data.frame(x=mp$x, y=mp$y, seg=missed, tp=0.5), L=L)
-    valmid <- f[mp, drop=FALSE]
-    seg <- c(seg, factor(missed, levels=1:ns))
+    #' nearest pixel value
+    valmid <- safelookup(f, mp)
+    #' concatenate factors
+    seg <- unlist(list(seg, factor(missed, levels=1:ns)))
     vals <- c(vals, valmid)
+    #' update
+    nper <- table(seg)
   }
   #' take average of data on each segment
-  mu <- as.numeric(by(vals, seg, mean, ..., na.rm=TRUE))
-  mu[is.na(mu)] <- 0
+  ##  mu <- as.numeric(by(vals, seg, mean, ..., na.rm=TRUE))
+  ## mu[is.na(mu)] <- 0
+  num <- tapplysum(as.numeric(vals), list(seg), na.rm=TRUE)
+  mu <- num/nper
   #' weighted sum
   len <- lengths.psp(as.psp(L))
   if(anyNA(vals)) {
-    p <- as.numeric(by(!is.na(vals), seg, mean, ..., na.rm=TRUE))
-    p[is.na(p)] <- 0
+    ##    p <- as.numeric(by(!is.na(vals), seg, mean, ..., na.rm=TRUE))
+    ##    p[is.na(p)] <- 0
+    defined <- as.numeric(!is.na(vals))
+    pnum <- tapplysum(defined, list(seg), na.rm=FALSE)
+    p <- pnum/nper
     len <- len * p
   }
   return(sum(mu * len))
@@ -713,6 +799,7 @@ as.data.frame.linim <- function(x, ...) {
 
 pairs.linim <- function(..., plot=TRUE, eps=NULL) {
   argh <- list(...)
+  cl <- match.call()
   ## unpack single argument which is a list of images
   if(length(argh) == 1) {
     arg1 <- argh[[1L]]
@@ -731,10 +818,19 @@ pairs.linim <- function(..., plot=TRUE, eps=NULL) {
   islinim <- sapply(imlist, inherits, what="linim")
   if(!any(islinim)) # shouldn't be here
     return(pairs.im(argh, plot=plot))
-  ## adjust names
-  imnames <- names(imlist) %orifnull% rep("", length(imlist))
-  if(any(needsname <- !nzchar(imnames))) 
-    imnames[needsname] <- paste0("V", seq_len(nim)[needsname])
+  ## determine image names for plotting
+  imnames <- argh$labels %orifnull% names(imlist)
+  if(length(imnames) != nim || !all(nzchar(imnames))) {
+    #' names not given explicitly
+    callednames <- paste(cl)[c(FALSE, isim, FALSE)]
+    backupnames <- paste0("V", seq_len(nim))
+    if(length(callednames) != nim) {
+      callednames <- backupnames
+    } else if(any(toolong <- (nchar(callednames) > 15))) {
+      callednames[toolong] <- backupnames[toolong]
+    }
+    imnames <- good.names(imnames, good.names(callednames, backupnames))
+  }
   names(imlist) <- imnames
   ## choose resolution
   if(is.null(eps)) {
@@ -762,7 +858,7 @@ pairs.linim <- function(..., plot=TRUE, eps=NULL) {
       do.call(hist.default,
               resolve.defaults(list(x=pixdf[,1L]),
                                rest,
-                               list(main=paste("Histogram of", imnames[1L]),
+                               list(xname=imnames[1L],
                                     xlab=imnames[1L])))
     }
   }
