@@ -3,10 +3,21 @@
 ##
 ## Fast versions of min(nndist(X)), max(nndist(X))
 ##
-##  $Revision: 1.4 $  $Date: 2014/10/24 00:22:30 $
+##  $Revision: 1.8 $  $Date: 2020/01/05 01:26:42 $
 
-minnndist <- function(X, positive=FALSE) {
+minnndist <- function(X, positive=FALSE, by=NULL) {
   stopifnot(is.ppp(X))
+  if(!is.null(by)) {
+    stopifnot(length(by) == npoints(X))
+    if(positive) {
+      retain <- !duplicated(X)
+      X <- X[retain]
+      by <- by[retain]
+    }
+    nn <- nndist(X, by=by)
+    result <- aggregate(nn, by=list(from=by), min, drop=FALSE)[,-1,drop=FALSE]
+    return(result)
+  }
   n <- npoints(X)
   if(n <= 1) return(NA)
   x <- X$x
@@ -19,20 +30,33 @@ minnndist <- function(X, positive=FALSE) {
               x = as.double(x[o]),
               y = as.double(y[o]),
               as.double(big),
-              result = as.double(numeric(1)))
+              result = as.double(numeric(1)),
+              PACKAGE = "spatstat")
   } else {
       z <- .C("minnnd2",
               n = as.integer(n),
               x = as.double(x[o]),
               y = as.double(y[o]),
               as.double(big),
-              result = as.double(numeric(1)))
+              result = as.double(numeric(1)),
+              PACKAGE = "spatstat")
   }
   return(sqrt(z$result))
 }
 
-maxnndist <- function(X, positive=FALSE) {
+maxnndist <- function(X, positive=FALSE, by=NULL) {
   stopifnot(is.ppp(X))
+  if(!is.null(by)) {
+    stopifnot(length(by) == npoints(X))
+    if(positive) {
+      retain <- !duplicated(X)
+      X <- X[retain]
+      by <- by[retain]
+    }
+    nn <- nndist(X, by=by)
+    result <- aggregate(nn, by=list(from=by), max, drop=FALSE)[,-1,drop=FALSE]
+    return(result)
+  }
   n <- npoints(X)
   if(n <= 1) return(NA)
   x <- X$x
@@ -45,16 +69,17 @@ maxnndist <- function(X, positive=FALSE) {
               x = as.double(x[o]),
               y = as.double(y[o]),
               as.double(big),
-              result = as.double(numeric(1)))
+              result = as.double(numeric(1)),
+              PACKAGE = "spatstat")
   } else {
       z <- .C("maxnnd2",
               n = as.integer(n),
               x = as.double(x[o]),
               y = as.double(y[o]),
               as.double(big),
-              result = as.double(numeric(1)))
+              result = as.double(numeric(1)),
+              PACKAGE = "spatstat")
   }
   return(sqrt(z$result))
 }
 
-          
