@@ -2,7 +2,7 @@
 #
 #    ord.S
 #
-#    $Revision: 1.4 $	$Date: 2007/01/11 03:36:02 $
+#    $Revision: 1.9 $	$Date: 2020/01/26 04:32:04 $
 #
 #    Ord process with user-supplied potential
 #
@@ -14,18 +14,17 @@
 # -------------------------------------------------------------------
 #	
 
-Ord <- function(pot, name) {
-  if(missing(name))
-    name <- "Ord process with user-defined potential"
-  
-  out <- 
+Ord <- local({
+
+  BlankOrd <- 
   list(
-         name     = name,
+         name     = "Ord process with user-defined potential",
          creator  = "Ord",
-         family    = ord.family,
-         pot      = pot,
+         family    = "ord.family",
+         pot      = NULL,
          par      = NULL,
          parnames = NULL,
+         hasInf   = NA,
          init     = NULL,
          update   = function(self, ...){
            do.call(Ord,
@@ -37,8 +36,19 @@ Ord <- function(pot, name) {
            print(self$pot)
            invisible()
          },
-       version=versionstring.spatstat()
+       version=NULL
   )
-  class(out) <- "interact"
-  return(out)
-}
+  class(BlankOrd) <- "interact"
+
+  Ord <- function(pot, name) {
+    out <- instantiate.interact(BlankOrd)
+    out$pot <- pot
+    if(!missing(name)) out$name <- name
+    return(out)
+  }
+
+  Ord <- intermaker(Ord, BlankOrd)
+})
+
+
+  
