@@ -1,7 +1,7 @@
 #
 #   pcfinhom.R
 #
-#   $Revision: 1.21 $   $Date: 2017/06/05 10:31:58 $
+#   $Revision: 1.23 $   $Date: 2019/05/24 10:24:48 $
 #
 #   inhomogeneous pair correlation function of point pattern 
 #
@@ -34,9 +34,11 @@ pcfinhom <- function(X, lambda=NULL, ..., r=NULL,
                              trans="translate",
                              translate="translate",
                              translation="translate",
+                             good="good",
                              best="best"),
                            multi=TRUE)
-
+  if("good" %in% correction)
+    correction[correction == "good"] <- good.correction.K(X)
   correction <- implemented.for.K(correction, win$type, correction.given)
 
   divisor <- match.arg(divisor)
@@ -96,12 +98,9 @@ pcfinhom <- function(X, lambda=NULL, ..., r=NULL,
         if(is.ppm(model)) {
           model <- update(model, Q=X)
           lambda <- fitted(model, dataonly=TRUE, leaveoneout=leaveoneout)
-        } else if(is.kppm(model)) {
-          model <- update(model, X=X)
-          lambda <- fitted(model, dataonly=TRUE, leaveoneout=leaveoneout)
         } else {
           model <- update(model, X=X)
-          lambda <- fitted(model, dataonly=TRUE)
+          lambda <- fitted(model, dataonly=TRUE, leaveoneout=leaveoneout)
         }
         danger <- FALSE
         if(miss.update) 
