@@ -1,7 +1,7 @@
 #
 #	plot.ppp.R
 #
-#	$Revision: 1.93 $	$Date: 2017/09/13 09:52:28 $
+#	$Revision: 1.95 $	$Date: 2018/03/22 00:46:59 $
 #
 #
 #--------------------------------------------------------------------------
@@ -136,7 +136,7 @@ plot.ppp <- local({
     }
     ##  ...........  non-numeric marks .........................
     um <- marklevels %orifnull%
-          if(is.factor(marx)) levels(marx) else sort(unique(marx))
+          if(is.factor(marx)) levels(marx) else sortunique(marx)
     ntypes <- length(um)
     if(!is.null(cols))
       cols <- rep.int(cols, ntypes)[1:ntypes]
@@ -190,11 +190,11 @@ plot.ppp <- local({
   type <- match.arg(type)
   if(missing(legend)) legend <- (type == "p")
 
-  if(!missing(maxsize) || !missing(markscale) || !missing(meansize))
-    warn.once("circlescale",
-              "Interpretation of arguments maxsize and markscale",
-              "has changed (in spatstat version 1.37-0 and later).",
-              "Size of a circle is now measured by its diameter.")
+#  if(!missing(maxsize) || !missing(markscale) || !missing(meansize))
+#    warn.once("circlescale",
+#              "Interpretation of arguments maxsize and markscale",
+#              "has changed (in spatstat version 1.37-0 and later).",
+#              "Size of a circle is now measured by its diameter.")
 
   if(clipped <- !is.null(clipwin)) {
     stopifnot(is.owin(clipwin))
@@ -333,7 +333,7 @@ plot.ppp <- local({
   legend <- legend && (symbolmaptype(symap) != "constant") 
   if(legend) {
     ## guess maximum size of symbols
-    maxsize <- invoke.symbolmap(symap, marx,
+    maxsize <- invoke.symbolmap(symap, symbolmapdomain(symap),
                                 corners(as.rectangle(x)),
                                 add=add, do.plot=FALSE)
     sizeguess <- if(maxsize <= 0) NULL else (1.5 * maxsize)
