@@ -1,7 +1,7 @@
 #
 #  quadratcount.R
 #
-#  $Revision: 1.46 $  $Date: 2014/03/17 10:20:09 $
+#  $Revision: 1.57 $  $Date: 2016/08/15 03:05:15 $
 #
 
 quadratcount <- function(X, ...) {
@@ -9,7 +9,7 @@ quadratcount <- function(X, ...) {
 }
 
 quadratcount.splitppp <- function(X, ...) {
-  as.listof(lapply(X, quadratcount, ...))
+  solapply(X, quadratcount, ...)
 }
 
 quadratcount.ppp <- function(X, nx=5, ny=nx, ...,
@@ -34,11 +34,12 @@ quadratcount.ppp <- function(X, nx=5, ny=nx, ...,
                        nx=nx, ny=ny, xbreaks=xbreaks, ybreaks=ybreaks,
                        keepempty=TRUE)
       # now delete the empty quadrats and the corresponding counts
-      nonempty <- !unlist(lapply(tiles(tess), is.empty))
+      nonempty <- !tiles.empty(tess)
+#     WAS: nonempty <- !unlist(lapply(tiles(tess), is.empty))
       if(!any(nonempty))
         stop("All tiles are empty")
       if(!all(nonempty)) {
-        ntiles <- sum(nonempty)
+#        ntiles <- sum(nonempty)
         tess   <- tess[nonempty]
         Xcount <- t(Xcount)[nonempty]
         # matrices and tables are in row-major order,
@@ -50,15 +51,18 @@ quadratcount.ppp <- function(X, nx=5, ny=nx, ...,
     }
   } else {
     # user-supplied tessellation
-    if(!inherits(tess, "tess"))
-      stop("The argument tess should be a tessellation", call.=FALSE)
+    if(!inherits(tess, "tess")) {
+      tess <- try(as.tess(tess), silent=TRUE)
+      if(inherits(tess, "try-error"))
+        stop("The argument tess should be a tessellation", call.=FALSE)
+    }
     if(tess$type == "rect") {
       # fast code for counting points in rectangular grid
       Xcount <- rectquadrat.countEngine(X$x, X$y, tess$xgrid, tess$ygrid)
     } else {
       # quadrats are another type of tessellation
       Y <- cut(X, tess)
-      if(any(is.na(marks(Y))))
+      if(anyNA(marks(Y)))
         warning("Tessellation does not contain all the points of X")
       Xcount <- table(tile=marks(Y))
     }
@@ -80,7 +84,7 @@ plot.quadratcount <- function(x, ...,
   # add=TRUE,  show.tiles=FALSE => plot numbers
   if(show.tiles || !add) {
     context <- if(show.tiles) tess else as.owin(tess)
-    do.call("plot",
+    do.call(plot,
             resolve.defaults(list(context, add=add),
                              list(...),
                              list(main=xname),
@@ -90,14 +94,15 @@ plot.quadratcount <- function(x, ...,
     labels <- paste(as.vector(entries))
     til <- tiles(tess)
     incircles <- lapply(til, incircle)
-    x0 <- unlist(lapply(incircles, function(z) { z$x }))
-    y0 <- unlist(lapply(incircles, function(z) { z$y }))
-    ra <- unlist(lapply(incircles, function(z) { z$r }))
-    do.call.matched("text.default",
+    x0 <- sapply(incircles, getElement, name="x")
+    y0 <- sapply(incircles, getElement, name="y")
+    ra <- sapply(incircles, getElement, name="r")
+    do.call.matched(text.default,
                     resolve.defaults(list(x=x0 + dx * ra, y = y0 + dy * ra),
                                      list(labels=labels),
                                      textargs, 
-                                     list(...)))
+                                     list(...)),
+                    funargs=graphicsPars("text"))
   }
   return(invisible(NULL))
 }
@@ -117,19 +122,24 @@ rectquadrat.breaks <- function(xr, yr, nx=5, ny=nx, xbreaks=NULL, ybreaks=NULL)
 rectquadrat.countEngine <- function(x, y, xbreaks, ybreaks, weights) {
   if(length(x) > 0) {
     # check validity of breaks
-    if(min(x) < min(xbreaks) || max(x) > max(xbreaks))
+    if(!all(inside.range(range(x), range(xbreaks))))
       stop("xbreaks do not span the actual range of x coordinates in data")
-    if(min(y) < min(ybreaks) || max(y) > max(ybreaks))
+    if(!all(inside.range(range(y), range(ybreaks))))
       stop("ybreaks do not span the actual range of y coordinates in data")
   }
-  xg <- cut(x, breaks=xbreaks, include.lowest=TRUE)
-  yg <- cut(y, breaks=ybreaks, include.lowest=TRUE)
-  if(missing(weights)) 
+  # WAS: 
+  # xg <- cut(x, breaks=xbreaks, include.lowest=TRUE)
+  # yg <- cut(y, breaks=ybreaks, include.lowest=TRUE)
+  xg <- fastFindInterval(x, xbreaks, labels=TRUE)
+  yg <- fastFindInterval(y, ybreaks, labels=TRUE)
+  if(missing(weights)) {
     sumz <- table(list(y=yg, x=xg))
-  else {
-    sumz <- tapply(weights, list(y=yg, x=xg), sum)
-    if(any(nbg <- is.na(sumz)))
-      sumz[nbg] <- 0
+  } else {
+    # was: 
+    # sumz <- tapply(weights, list(y=yg, x=xg), sum)
+    # if(any(nbg <- is.na(sumz)))
+    #  sumz[nbg] <- 0
+    sumz <- tapplysum(weights, list(y=yg, x=xg), do.names=TRUE)
   }
   # reverse order of y 
   sumz <- sumz[rev(seq_len(nrow(sumz))), ]
@@ -147,7 +157,7 @@ quadrats <- function(X, nx=5, ny=nx, xbreaks = NULL, ybreaks = NULL,
   yr <- W$yrange
   b <- rectquadrat.breaks(xr, yr, nx, ny, xbreaks, ybreaks)
   # rectangular tiles
-  Z <- tess(xgrid=b$xbreaks, ygrid=b$ybreaks)
+  Z <- tess(xgrid=b$xbreaks, ygrid=b$ybreaks, unitname=unitname(W))
   if(W$type != "rectangle") {
     # intersect rectangular tiles with window W
     if(!keepempty) {
@@ -170,6 +180,8 @@ as.owin.quadratcount <- function(W, ..., fatal=TRUE) {
   return(as.owin(as.tess(W), ..., fatal=fatal))
 }
 
+domain.quadratcount <- Window.quadratcount <- function(X, ...) { as.owin(X) }
+
 intensity.quadratcount <- function(X, ..., image=FALSE) {
   Y <- as.tess(X)
   a <- tile.areas(Y)
@@ -191,7 +203,7 @@ intensity.quadratcount <- function(X, ..., image=FALSE) {
 }
 
 ## The shift method is undocumented.
-## It is only needed in plot.listof
+## It is only needed in plot.listof / plot.solist / plot.layered
 
 shift.quadratcount <- function(X, ...) {
   attr(X, "tess") <- te <- shift(attr(X, "tess"), ...)
