@@ -5,18 +5,48 @@
 #         resid1plot       one or more unrelated individual plots 
 #         resid1panel      one panel of resid1plot
 #
-#   $Revision: 1.26 $    $Date: 2014/05/09 01:12:42 $
+#   $Revision: 1.37 $    $Date: 2019/12/15 05:26:58 $
 #
 #
 
-resid4plot <-
-  function(RES,
-           plot.neg=c("image", "discrete", "contour", "imagecontour"),
-           plot.smooth=c("imagecontour", "image", "contour", "persp"),
-           spacing=0.1, srange=NULL, monochrome=FALSE, main=NULL,
-           ...)
+resid4plot <- local({
+
+  Contour <- function(...,
+                      pch, chars, cols, etch, size,
+                      maxsize, meansize, markscale, symap, zap,
+                      legend, leg.side, leg.args) {
+    ## avoid passing arguments of plot.ppp to contour.default
+    contour(...)
+  }
+
+  do.clean <- function(fun, ..., 
+                       pch, chars, cols, etch, size,
+                       maxsize, meansize, markscale, symap, zap,
+                       legend, leg.side, leg.args,
+                       nlevels, levels, labels, drawlabels, labcex) {
+    ## avoid passing arguments of plot.ppp, contour.default to other functions
+    do.call(fun, list(...)) 
+  }
+
+  do.lines <- function(x, y, defaulty=1, ...) {
+    do.call(lines,
+            resolve.defaults(list(x, y),
+                             list(...),
+                             list(lty=defaulty)))
+  }
+
+  resid4plot <-
+    function(RES,
+             plot.neg=c("image", "discrete", "contour", "imagecontour"),
+             plot.smooth=c("imagecontour", "image", "contour", "persp"),
+             spacing=0.1, outer=3, srange=NULL, monochrome=FALSE, main=NULL,
+             xlab="x coordinate", ylab="y coordinate", rlab,
+             col.neg=NULL, col.smooth=NULL,
+             ...)
 {
   plot.neg <- match.arg(plot.neg)
+  if(missing(rlab)) rlab <- NULL
+  rlablines <- if(is.null(rlab)) 1 else sum(nzchar(rlab))
   clip     <- RES$clip
   Yclip    <- RES$Yclip
   Z        <- RES$smooth$Z
@@ -32,21 +62,22 @@ resid4plot <-
   space <- spacing * max(wide,high)
   width <- wide + space + wide
   height <- high + space + high
-  outerspace <- 3 * space
-  plot(c(0, width) + outerspace * c(-1,1),
-       c(0, height) + outerspace * c(-1,1),
+  outerspace <- outer * space
+  outerRspace <- (outer - 1 + rlablines) * space
+  plot(c(0, width) + c(-outerRspace, outerspace),
+       c(0, height) + c(-outerspace, outerRspace),
        type="n", asp=1.0, axes=FALSE, xlab="", ylab="")
-  # determine colour map
+  # determine colour map for background
+  nullvalue <- if(type == "eem") 1 else 0
   if(is.null(srange)) {
     Yrange <- if(!is.null(Ydens)) summary(Ydens)$range else NULL
     Zrange <- if(!is.null(Z)) summary(Z)$range else NULL
-    srange <- range(c(0, Yrange, Zrange), na.rm=TRUE)
-  } else {
-    stopifnot(is.numeric(srange) && length(srange) == 2)
-    stopifnot(all(is.finite(srange)))
-  }
-  cols <- beachcolours(srange, if(type=="eem") 1 else 0, monochrome)
-                      
+    srange <- range(c(Yrange, Zrange, nullvalue), na.rm=TRUE)
+  } else check.range(srange)
+  backcols <- beachcolours(srange, nullvalue, monochrome)
+  if(is.null(col.neg)) col.neg <- backcols
+  if(is.null(col.smooth)) col.smooth <- backcols
+  
   # ------ plot residuals/marks (in top left panel) ------------
   Xlowleft <- c(W$xrange[1],W$yrange[1])
   vec <- c(0, high) + c(0, space) - Xlowleft
@@ -62,9 +93,9 @@ resid4plot <-
   # pre-plot the window(s)
   if(!redundant) {
     if(!clip) 
-      plot(Ys$window, add=TRUE, ...)
+      do.clean(plot, Ys$window, add=TRUE, ...)
     else
-      ploterodewin(Ws, Ys$window, add=TRUE, ...)
+      do.clean(ploterodewin, Ws, Ys$window, add=TRUE, ...)
   }
 
   ## adjust position of legend associated with eroded window
@@ -77,24 +108,25 @@ resid4plot <-
          discrete={
            neg <- (Ys$marks < 0)
            ## plot negative masses of discretised measure as squares
-           if(any(c("maxsize", "markscale") %in% names(list(...))))
-             plot(Ys[neg], add=TRUE, ..., legend=FALSE)
-           else {
-             hackmax <- 0.5 * sqrt(area.owin(Wclip)/Yclip$n)
-             plot(Ys[neg], add=TRUE, maxsize=hackmax, ..., legend=FALSE)
+           if(any(c("maxsize","meansize","markscale") %in% names(list(...)))) {
+             plot(Ys[neg], add=TRUE, legend=FALSE, ...)
+           } else {
+             hackmax <- 0.5 * sqrt(area(Wclip)/Yclip$n)
+             plot(Ys[neg], add=TRUE, legend=FALSE, maxsize=hackmax, ...)
            }
            ## plot positive masses at atoms
            plot(Ys[!neg], add=TRUE,
                 leg.side="left", leg.args=list(sep=sep),
-                show.all=TRUE, main="", ...)
+                show.all=TRUE, main="",
+                ...)
          },
          contour = {
            Yds <- shift(Ydens, vec)
            Yms <- shift(Ymass, vec)
-           contour(Yds, add=TRUE, ...)
-           do.call("plot",
+           Contour(Yds, add=TRUE, ...)
+           do.call(plot,
                    resolve.defaults(list(x=Yms, add=TRUE),
-                                    list(...),
+                                    list(...), 
                                     list(use.marks=showscale,
                                          leg.side="left", show.all=TRUE,
                                          main="", leg.args=list(sep=sep))))
@@ -104,13 +136,18 @@ resid4plot <-
            Yds <- shift(Ydens, vec)
            Yms <- shift(Ymass, vec)
            if(redundant)
-             ploterodeimage(Ws, Yds, rangeZ=srange, colsZ=cols, ...)
-           else if(type != "eem") 
-             image(Yds, add=TRUE, ribbon=FALSE, col=cols, zlim=srange, ...)
+             do.clean(ploterodeimage,
+                      Ws, Yds, rangeZ=srange, colsZ=col.neg,
+                      ...)
+           else if(type != "eem")
+             do.clean(image,
+                      Yds, add=TRUE, ribbon=FALSE,
+                      col=col.neg, zlim=srange,
+                      ...)
            if(plot.neg == "imagecontour")
-             contour(Yds, add=TRUE, ...)
+             Contour(Yds, add=TRUE, ...)
            ## plot positive masses at atoms
-           do.call("plot",
+           do.call(plot,
                    resolve.defaults(list(x=Yms, add=TRUE),
                                     list(...),
                                     list(use.marks=showscale,
@@ -123,23 +160,24 @@ resid4plot <-
   Zs <- shift.im(Z, vec)
   switch(plot.smooth,
          image={
-           image(Zs, add=TRUE, col=cols, zlim=srange, ribbon=FALSE, ...)},
-         contour={contour(Zs, add=TRUE, ...)},
+           do.clean(image,
+                    Zs, add=TRUE, col=col.smooth,
+                    zlim=srange, ribbon=FALSE,
+                    ...)
+         },
+         contour={
+           Contour(Zs, add=TRUE, ...)
+         },
          persp={ warning("persp not available in 4-panel plot") },
          imagecontour={
-             image(Zs, add=TRUE, col=cols, zlim=srange, ribbon=FALSE, ...)
-             contour(Zs, add=TRUE, ...)
+             do.clean(image,
+                      Zs, add=TRUE, col=col.smooth, zlim=srange, ribbon=FALSE,
+                      ...)
+             Contour(Zs, add=TRUE, ...)
            }
-         )  
+         )
   lines(Zs$xrange[c(1,2,2,1,1)], Zs$yrange[c(1,1,2,2,1)])
   # -------------- lurking variable plots -----------------------
-  do.lines <-
-    function(x, y, defaulty=1, ...) {
-      do.call("lines",
-              resolve.defaults(list(x, y),
-                               list(...),
-                               list(lty=defaulty)))
-    }
   # --------- lurking variable plot for x coordinate ------------------
   #           (cumulative or marginal)
   #           in bottom left panel
@@ -149,8 +187,8 @@ resid4plot <-
     observedX <-    a$x
     theoreticalV <- a$ExZ
     theoreticalX <- a$x
-    theoreticalSD <- NULL
-    ylabel <- paste("marginal of", typename)
+    theoreticalSD <- theoreticalHI <- theoreticalLO <- NULL
+    if(is.null(rlab)) rlab <- paste("marginal of", typename)
   } else if(!is.null(RES$xcumul)) {
     a <- RES$xcumul
     observedX <- a$empirical$covariate
@@ -158,34 +196,45 @@ resid4plot <-
     theoreticalX <- a$theoretical$covariate
     theoreticalV <- a$theoretical$mean
     theoreticalSD <- a$theoretical$sd
-    ylabel <- paste("cumulative sum of", typename)
+    theoreticalHI <- a$theoretical$upper
+    theoreticalLO <- a$theoretical$lower
+    if(is.null(rlab)) rlab <- paste("cumulative sum of", typename)
   }
   # pretty axis marks
   pX <- pretty(theoreticalX)
-  if(is.null(theoreticalSD))
-    pV <- pretty(c(0,observedV,theoreticalV))
-  else
-    pV <- pretty(c(0,observedV,theoreticalV,
-                   theoreticalV+2*theoreticalSD,
-                   theoreticalV-2*theoreticalSD))
+  rV <- range(0, observedV, theoreticalV, theoreticalHI, theoreticalLO)
+  if(!is.null(theoreticalSD))
+    rV <- range(rV,
+                theoreticalV+2*theoreticalSD,
+                theoreticalV-2*theoreticalSD)
+  pV <- pretty(rV)
   # rescale smoothed values
   rr <- range(c(0, observedV, theoreticalV, pV))
   yscale <- function(y) { high * (y - rr[1])/diff(rr) }
   xscale <- function(x) { x - W$xrange[1] }
-  do.lines(xscale(observedX), yscale(observedV), 1, ...)
-  do.lines(xscale(theoreticalX), yscale(theoreticalV), 2, ...)
+  if(!is.null(theoreticalHI)) 
+    do.call.matched(polygon,
+                    resolve.defaults(
+                      list(x=xscale(c(theoreticalX, rev(theoreticalX))),
+                           y=yscale(c(theoreticalHI, rev(theoreticalLO)))),
+                      list(...),
+                      list(col="grey", border=NA)))
+  do.clean(do.lines, xscale(observedX), yscale(observedV), 1, ...)
+  do.clean(do.lines, xscale(theoreticalX), yscale(theoreticalV), 2, ...)
   if(!is.null(theoreticalSD)) {
-    do.lines(xscale(theoreticalX),
+    do.clean(do.lines,
+             xscale(theoreticalX),
              yscale(theoreticalV + 2 * theoreticalSD),
              3, ...)
-    do.lines(xscale(theoreticalX),
+    do.clean(do.lines,
+             xscale(theoreticalX),
              yscale(theoreticalV - 2 * theoreticalSD),
              3, ...)
   }
   axis(side=1, pos=0, at=xscale(pX), labels=pX)
-  text(xscale(mean(theoreticalX)), - outerspace, "x coordinate")
+  text(xscale(mean(theoreticalX)), - outerspace, xlab)
   axis(side=2, pos=0, at=yscale(pV), labels=pV)
-  text(-outerspace, yscale(mean(pV)), ylabel, srt=90)
+  text(-outerRspace, yscale(mean(pV)), rlab, srt=90)
   
   # --------- lurking variable plot for y coordinate ------------------
   #           (cumulative or marginal)
@@ -197,7 +246,7 @@ resid4plot <-
     theoreticalV <- a$EyZ
     theoreticalY <- a$y
     theoreticalSD <- NULL
-    ylabel <- paste("marginal of", typename)
+    if(is.null(rlab)) rlab <- paste("marginal of", typename)
   } else if(!is.null(RES$ycumul)) {
     a <- RES$ycumul
     observedV <- a$empirical$value
@@ -205,52 +254,86 @@ resid4plot <-
     theoreticalY <- a$theoretical$covariate
     theoreticalV <- a$theoretical$mean
     theoreticalSD <- a$theoretical$sd
-    ylabel <- paste("cumulative sum of", typename)
+    theoreticalHI <- a$theoretical$upper
+    theoreticalLO <- a$theoretical$lower
+    if(is.null(rlab)) rlab <- paste("cumulative sum of", typename)
   }
   # pretty axis marks
   pY <- pretty(theoreticalY)
-  if(is.null(theoreticalSD))
-    pV <- pretty(c(0,observedV,theoreticalV))
-  else
-    pV <- pretty(c(0,observedV,theoreticalV,
-                   theoreticalV+2*theoreticalSD,
-                   theoreticalV-2*theoreticalSD))
+  rV <- range(0, observedV, theoreticalV, theoreticalHI, theoreticalLO)
+  if(!is.null(theoreticalSD))
+    rV <- range(rV,
+                theoreticalV+2*theoreticalSD,
+                theoreticalV-2*theoreticalSD)
+  pV <- pretty(rV)
   # rescale smoothed values
   rr <- range(c(0, observedV, theoreticalV, pV))
   yscale <- function(y) { y - W$yrange[1] + high + space}
   xscale <- function(x) { wide + space + wide * (rr[2] - x)/diff(rr) }
-  do.lines(xscale(observedV), yscale(observedY), 1, ...)
-  do.lines(xscale(theoreticalV), yscale(theoreticalY), 2, ...)
+  if(!is.null(theoreticalHI)) 
+    do.call.matched(polygon,
+                    resolve.defaults(
+                      list(x=xscale(c(theoreticalHI, rev(theoreticalLO))),
+                           y=yscale(c(theoreticalY,  rev(theoreticalY)))),
+                      list(...),
+                      list(col="grey", border=NA)))
+  do.clean(do.lines, xscale(observedV), yscale(observedY), 1, ...)
+  do.clean(do.lines, xscale(theoreticalV), yscale(theoreticalY), 2, ...)
   if(!is.null(theoreticalSD)) {
-    do.lines(xscale(theoreticalV+2*theoreticalSD),
+    do.clean(do.lines,
+             xscale(theoreticalV+2*theoreticalSD),
              yscale(theoreticalY),
              3, ...)
-    do.lines(xscale(theoreticalV-2*theoreticalSD),
+    do.clean(do.lines,
+             xscale(theoreticalV-2*theoreticalSD),
              yscale(theoreticalY),
              3, ...)
   }
   axis(side=4, pos=width, at=yscale(pY), labels=pY)
-  text(width + outerspace, yscale(mean(theoreticalY)), "y coordinate", srt=90)
+  text(width + outerspace, yscale(mean(theoreticalY)), ylab, srt=90)
   axis(side=3, pos=height, at=xscale(pV), labels=pV)
-  text(xscale(mean(pV)), height + outerspace, ylabel)
+  text(xscale(mean(pV)), height + outerRspace, rlab)
   #
   if(!is.null(main))
     title(main=main)
   invisible(NULL)
 }
 
+  resid4plot
+})
+
 #
 #
 #   Residual plot: single panel(s)
 #
 #
 
-resid1plot <-
+resid1plot <- local({
+
+  Contour <- function(...,
+                      pch, chars, cols, etch, size,
+                      maxsize, meansize, markscale, symap, zap,
+                      legend, leg.side, leg.args) {
+    ## avoid passing arguments of plot.ppp to contour.default
+    contour(...)
+  }
+
+  do.clean <- function(fun, ..., 
+                       pch, chars, cols, etch, size,
+                       maxsize, meansize, markscale, symap, zap,
+                       legend, leg.side, leg.args,
+                       nlevels, levels, labels, drawlabels, labcex) {
+    ## avoid passing arguments of plot.ppp, contour.default to other functions
+    do.call(fun, list(...)) 
+  }
+
+  resid1plot <- 
   function(RES, opt,
            plot.neg=c("image", "discrete", "contour", "imagecontour"),
            plot.smooth=c("imagecontour", "image", "contour", "persp"),
            srange=NULL, monochrome=FALSE, main=NULL,
-           add=FALSE, show.all=!add, do.plot=TRUE, 
+           add=FALSE, show.all=!add, do.plot=TRUE,
+           col.neg=NULL, col.smooth=NULL, 
            ...) {
     if(!any(unlist(opt[c("all", "marks", "smooth",
                          "xmargin", "ymargin", "xcumul", "ycumul")])))
@@ -280,12 +363,15 @@ resid1plot <-
     Ymass <- RES$Ymass[Wclip]
     ## determine colour map
     if(opt$all || opt$marks || opt$smooth) {
+      nullvalue <- if(type == "eem") 1 else 0
       if(is.null(srange)) {
         Yrange <- if(!is.null(Ydens)) summary(Ydens)$range else NULL
         Zrange <- if(!is.null(Z)) summary(Z)$range else NULL
-        srange <- range(c(0, Yrange, Zrange), na.rm=TRUE)
-      }
-      cols <- beachcolours(srange, if(type=="eem") 1 else 0, monochrome)
+        srange <- range(c(Yrange, Zrange, nullvalue), na.rm=TRUE)
+      } else check.range(srange)
+      backcols <- beachcolours(srange, nullvalue, monochrome)
+      if(is.null(col.neg)) col.neg <- backcols
+      if(is.null(col.smooth)) col.smooth <- backcols
     }
     ## determine main heading
     if(is.null(main)) {
@@ -304,15 +390,18 @@ resid1plot <-
                    (type != "eem") && (Yclip$window$type == "mask")
       ## pre-plot the window(s)
       if(redundant && !add) {
-        z <- plot(as.rectangle(W), box=FALSE, main="",
-                  do.plot=do.plot, ...)
+        z <- do.clean(plot,
+                      as.rectangle(W), box=FALSE, main="",
+                      do.plot=do.plot, ...)
       } else {
         if(!clip) 
-          z <- plot(W, main="",
-               add=add, show.all=show.all, do.plot=do.plot, ...)
+          z <- do.clean(plot,
+                        W, main="",
+                        add=add, show.all=show.all, do.plot=do.plot, ...)
         else
-          z <- ploterodewin(W, Wclip, main="",
-                       add=add, show.all=show.all, do.plot=do.plot, ...)
+          z <- do.clean(ploterodewin,
+                        W, Wclip, main="",
+                        add=add, show.all=show.all, do.plot=do.plot, ...)
       }
       bb <- as.owin(z)
 
@@ -324,7 +413,7 @@ resid1plot <-
                  z <- plot(Y[neg], add=TRUE,
                           show.all=show.all, do.plot=do.plot, ...)
                } else {
-                 hackmax <- 0.5 * sqrt(area.owin(Wclip)/Yclip$n)
+                 hackmax <- 0.5 * sqrt(area(Wclip)/Yclip$n)
                  z <- plot(Y[neg], add=TRUE, maxsize=hackmax,
                            show.all=show.all, do.plot=do.plot, ...)
                }
@@ -334,30 +423,32 @@ resid1plot <-
                bb <- boundingbox(bb, z, zp)
            },
            contour = {
-             z <- contour(Ydens, add=TRUE, do.plot=do.plot, ...)
+             z <- Contour(Ydens, add=TRUE, do.plot=do.plot, ...)
              bb <- boundingbox(bb, z)
            },
            imagecontour=,
            image={
              if(redundant) {
-               z <- ploterodeimage(W, Ydens, rangeZ=srange, colsZ=cols,
-                                   add=add, show.all=show.all, main="", 
-                                   do.plot=do.plot, ...)
+               z <- do.clean(ploterodeimage,
+                             W, Ydens, rangeZ=srange, colsZ=col.neg,
+                             add=add, show.all=show.all, main="", 
+                             do.plot=do.plot, ...)
              } else if(type != "eem") {
-               z <- image(Ydens, col=cols, zlim=srange, ribbon=FALSE,
-                          add=TRUE, show.all=show.all, do.plot=do.plot,
-                          main="", ...)
+               z <- do.clean(image,
+                             Ydens, col=col.neg, zlim=srange, ribbon=FALSE,
+                             add=TRUE, show.all=show.all, do.plot=do.plot,
+                             main="", ...)
              }
              bb <- boundingbox(bb, z)
              if(plot.neg == "imagecontour") {
-               z <- contour(Ydens, add=TRUE,
+               z <- Contour(Ydens, add=TRUE,
                             show.all=show.all, do.plot=do.plot, ...)
                bb <- boundingbox(bb, z)
              }
              ## decide whether mark scale should be shown
              showscale <- (type != "raw")
              ## plot positive masses at atoms
-             z <- do.call("plot",
+             z <- do.call(plot,
                           resolve.defaults(list(x=Ymass, add=TRUE),
                                            list(...),
                                            list(use.marks=showscale,
@@ -372,26 +463,29 @@ resid1plot <-
     if(!clip) {
       switch(plot.smooth,
            image={
-             z <- image(Z, main="", axes=FALSE, xlab="", ylab="",
-                        col=cols, zlim=srange, ribbon=FALSE,
-                        do.plot=do.plot, add=add, show.all=show.all, ...)
+             z <- do.clean(image,
+                           Z, main="", axes=FALSE, xlab="", ylab="",
+                           col=col.smooth, zlim=srange, ribbon=FALSE,
+                           do.plot=do.plot, add=add, show.all=show.all, ...)
              bb <- as.owin(z)
            },
            contour={
-             z <- contour(Z, main="", axes=FALSE, xlab="", ylab="",
-                        do.plot=do.plot, add=add, show.all=show.all, ...)
+             z <- Contour(Z, main="", axes=FALSE, xlab="", ylab="",
+                          do.plot=do.plot, add=add, show.all=show.all, ...)
              bb <- as.owin(z)
            },
            persp={
              if(do.plot)
-               persp(Z, main="", axes=FALSE, xlab="", ylab="", ...)
+               do.clean(persp,
+                        Z, main="", axes=FALSE, xlab="", ylab="", ...)
              bb <- NULL
            },
            imagecontour={
-             z <- image(Z, main="", axes=FALSE, xlab="", ylab="",
-                        col=cols, zlim=srange, ribbon=FALSE,
-                        do.plot=do.plot, add=add, show.all=show.all, ...)
-             contour(Z, add=TRUE, do.plot=do.plot, ...)
+             z <- do.clean(image,
+                           Z, main="", axes=FALSE, xlab="", ylab="",
+                           col=col.smooth, zlim=srange, ribbon=FALSE,
+                           do.plot=do.plot, add=add, show.all=show.all, ...)
+             Contour(Z, add=TRUE, do.plot=do.plot, ...)
              bb <- as.owin(z)
            }
              )
@@ -399,30 +493,36 @@ resid1plot <-
     } else {
       switch(plot.smooth,
              image={
-               plot(as.rectangle(W), box=FALSE, main=main,
-                    do.plot=do.plot, ...)
-               z <- ploterodeimage(W, Z, colsZ=cols, rangeZ=srange,
-                                   do.plot=do.plot, ...)
+               do.clean(plot,
+                        as.rectangle(W), box=FALSE, main=main,
+                        do.plot=do.plot, add=add, ...)
+               z <- do.clean(ploterodeimage,
+                             W, Z, colsZ=col.smooth, rangeZ=srange,
+                             do.plot=do.plot, ...)
                bb <- boundingbox(as.rectangle(W), z)
              },
              contour={
-               plot(W, main=main,
-                    do.plot=do.plot, add=add, show.all=show.all, ...)
-               z <- contour(Z, add=TRUE,
+               do.clean(plot,
+                        W, main=main,
+                        do.plot=do.plot, add=add, show.all=show.all, ...)
+               z <- Contour(Z, add=TRUE,
                             show.all=show.all, do.plot=do.plot, ...)
                bb <- as.owin(z)
              },
              persp={
                if(do.plot) 
-                 persp(Z, main=main, axes=FALSE, xlab="", ylab="", ...)
+                 do.clean(persp,
+                          Z, main=main, axes=FALSE, xlab="", ylab="", ...)
                bb <- NULL
              },
              imagecontour={
-               plot(as.rectangle(W), box=FALSE, main=main,
-                    do.plot=do.plot, ...)
-               z <- ploterodeimage(W, Z, colsZ=cols, rangeZ=srange,
-                                   do.plot=do.plot, ...)
-               contour(Z, add=TRUE, do.plot=do.plot, ...)
+               do.clean(plot,
+                        as.rectangle(W), box=FALSE, main=main,
+                        do.plot=do.plot, add=add, ...)
+               z <- do.clean(ploterodeimage,
+                             W, Z, colsZ=col.smooth, rangeZ=srange,
+                             do.plot=do.plot, ...)
+               Contour(Z, add=TRUE, do.plot=do.plot, ...)
                bb <- as.owin(z)
              }
              )
@@ -434,11 +534,12 @@ resid1plot <-
     a <- RES$xcumul
     obs <- a$empirical
     theo <- a$theoretical
-    resid1panel(obs$covariate, obs$value,
-               theo$covariate, theo$mean, theo$sd,
-               "x coordinate", "cumulative mark", main=main,
-                ...,
-                do.plot=do.plot)
+    do.clean(resid1panel,
+             obs$covariate, obs$value,
+             theo$covariate, theo$mean, theo$sd,
+             "x coordinate", "cumulative mark", main=main,
+             ...,
+             do.plot=do.plot)
     bb <- NULL
   }
   
@@ -447,29 +548,32 @@ resid1plot <-
     a <- RES$ycumul
     obs <- a$empirical
     theo <- a$theoretical
-    resid1panel(obs$covariate, obs$value,
-               theo$covariate, theo$mean, theo$sd,
-               "y coordinate", "cumulative mark", main=main,
-                ...,
-                do.plot=do.plot)
+    do.clean(resid1panel,
+             obs$covariate, obs$value,
+             theo$covariate, theo$mean, theo$sd,
+             "y coordinate", "cumulative mark", main=main,
+             ...,
+             do.plot=do.plot)
     bb <- NULL
   }
   ## ------------  x margin -----------------------------------------
   if(opt$xmargin) {
     a <- RES$xmargin
-    resid1panel(a$x, a$xZ, a$x, a$ExZ, NULL,
-               "x coordinate", "marginal of residuals", main=main,
-                ...,
-                do.plot=do.plot)
+    do.clean(resid1panel,
+             a$x, a$xZ, a$x, a$ExZ, NULL,
+             "x coordinate", "marginal of residuals", main=main,
+             ...,
+             do.plot=do.plot)
     bb <- NULL
   }
   # ------------  y margin -----------------------------------------
   if(opt$ymargin) {
     a <- RES$ymargin
-    resid1panel(a$y, a$yZ, a$y, a$EyZ, NULL,
-               "y coordinate", "marginal of residuals", main=main,
-                ...,
-                do.plot=do.plot)
+    do.clean(resid1panel,
+             a$y, a$yZ, a$y, a$EyZ, NULL,
+             "y coordinate", "marginal of residuals", main=main,
+             ...,
+             do.plot=do.plot)
     bb <- NULL
   }
 
@@ -477,6 +581,18 @@ resid1plot <-
   return(invisible(bb))
 }
 
+resid1plot
+})
+
+
+resid1panel <- local({
+
+  do.lines <- function(x, y, defaulty=1, ...) {
+      do.call(lines,
+              resolve.defaults(list(x, y),
+                               list(...),
+                               list(lty=defaulty)))
+  }
 
 resid1panel <- function(observedX, observedV,
                         theoreticalX, theoreticalV, theoreticalSD, xlab, ylab,
@@ -490,13 +606,6 @@ resid1panel <- function(observedX, observedV,
     rV <- range(c(rV, theoreticalV + 2*theoreticalSD,
                   theoreticalV - 2*theoreticalSD))
   ## argument handling
-  do.lines <-
-    function(x, y, defaulty=1, ...) {
-      do.call("lines",
-              resolve.defaults(list(x, y),
-                               list(...),
-                               list(lty=defaulty)))
-    }
   ## start plot
   plot(rX, rV, type="n", xlab=xlab, ylab=ylab, ...)
   do.lines(observedX, observedV, 1, ...)
@@ -507,6 +616,9 @@ resid1panel <- function(observedX, observedV,
   }
 }
 
+resid1panel
+})
+
 #
 #
 ploterodewin <- function(W1, W2, col.edge=grey(0.75), col.inside=rgb(1,0,0),
@@ -524,12 +636,13 @@ ploterodewin <- function(W1, W2, col.edge=grey(0.75), col.inside=rgb(1,0,0),
          },
          mask={
            Z <- as.im(W1)
-           x <- as.vector(raster.x(W1))
-           y <- as.vector(raster.y(W1))
+           x <- as.vector(rasterx.mask(W1))
+           y <- as.vector(rastery.mask(W1))
            ok <- inside.owin(x, y, W2)
            Z$v[ok] <- 2
            z <- plot(Z, ..., col=c(col.edge, col.inside),
-                     add=TRUE, ribbon=FALSE, do.plot=do.plot)
+                     add=TRUE, ribbon=FALSE, do.plot=do.plot,
+                     show.all=TRUE)
          }
          )
   return(z)
@@ -541,8 +654,9 @@ ploterodeimage <- function(W, Z, ..., Wcol=grey(0.75), rangeZ, colsZ,
   # Image Z is assumed to live on a subset of mask W
   # colsZ are the colours for the values in the range 'rangeZ'
 
-  if(W$type != "mask" && do.plot) {
-    plot(W, add=TRUE)
+  if(!is.mask(W)) {
+    if(do.plot)
+      plot(W, add=TRUE)
     W <- as.mask(W)
   }
   
@@ -560,8 +674,8 @@ ploterodeimage <- function(W, Z, ..., Wcol=grey(0.75), rangeZ, colsZ,
   # (with W-pixels initialised to Wvalue)
   X <- as.im(Wvalue, W)
   # Look up Z-values of W-pixels
-  xx <- as.vector(raster.x(W))
-  yy <- as.vector(raster.y(W))
+  xx <- as.vector(rasterx.mask(W))
+  yy <- as.vector(rastery.mask(W))
   Zvalues <- lookup.im(Z, xx, yy, naok = TRUE, strict=FALSE)
   # Overwrite pixels in Z
   inZ <- !is.na(Zvalues)
