@@ -1,12 +1,15 @@
-#
-#  rhohat.R
-#
-#  $Revision: 1.72 $  $Date: 2017/08/17 09:35:46 $
-#
-#  Non-parametric estimation of a transformation rho(z) determining
-#  the intensity function lambda(u) of a point process in terms of a
-#  spatial covariate Z(u) through lambda(u) = rho(Z(u)).
-#  More generally allows offsets etc.
+#'
+#'  rhohat.R
+#'
+#'  $Revision: 1.85 $  $Date: 2020/01/27 10:22:38 $
+#'
+#'  Non-parametric estimation of a transformation rho(z) determining
+#'  the intensity function lambda(u) of a point process in terms of a
+#'  spatial covariate Z(u) through lambda(u) = rho(Z(u)).
+#'  More generally allows offsets etc.
+
+#' Copyright (c) Adrian Baddeley 2015-2019
+#' GNU Public Licence GPL >= 2.0
 
 rhohat <- function(object, covariate, ...) {
   UseMethod("rhohat")
@@ -17,14 +20,16 @@ rhohat.ppp <- rhohat.quad <-
            baseline=NULL, weights=NULL,
            method=c("ratio", "reweight", "transform"),
            horvitz=FALSE,
-           smoother=c("kernel", "local"),
+           smoother=c("kernel", "local", "decreasing", "increasing"),
            subset=NULL,
            dimyx=NULL, eps=NULL,
            n=512, bw="nrd0", adjust=1, from=NULL, to=NULL, 
-           bwref=bw, covname, confidence=0.95) {
+           bwref=bw, covname, confidence=0.95, positiveCI) {
   callstring <- short.deparse(sys.call())
   smoother <- match.arg(smoother)
   method <- match.arg(method)
+  if(missing(positiveCI))
+    positiveCI <- (smoother == "local")
   if(missing(covname)) 
     covname <- sensiblevarname(short.deparse(substitute(covariate)), "X")
   if(is.null(adjust))
@@ -69,6 +74,7 @@ rhohat.ppp <- rhohat.quad <-
                n=n, bw=bw, adjust=adjust, from=from, to=to,
                bwref=bwref, covname=covname, covunits=covunits,
                confidence=confidence,
+               positiveCI=positiveCI, 
                modelcall=modelcall, callstring=callstring)
 }
 
@@ -76,14 +82,17 @@ rhohat.ppm <- function(object, covariate, ...,
                        weights=NULL,
                        method=c("ratio", "reweight", "transform"),
                        horvitz=FALSE,
-                       smoother=c("kernel", "local"),
+                       smoother=c("kernel", "local", "decreasing", "increasing"),
                        subset=NULL,
                        dimyx=NULL, eps=NULL,
                        n=512, bw="nrd0", adjust=1, from=NULL, to=NULL, 
-                       bwref=bw, covname, confidence=0.95) {
+                       bwref=bw, covname, confidence=0.95,
+                       positiveCI) {
   callstring <- short.deparse(sys.call())
   smoother <- match.arg(smoother)
   method <- match.arg(method)
+  if(missing(positiveCI))
+    positiveCI <- (smoother == "local")
   if(missing(covname)) 
     covname <- sensiblevarname(short.deparse(substitute(covariate)), "X")
   if(is.null(adjust))
@@ -125,7 +134,7 @@ rhohat.ppm <- function(object, covariate, ...,
                resolution=list(dimyx=dimyx, eps=eps),
                n=n, bw=bw, adjust=adjust, from=from, to=to,
                bwref=bwref, covname=covname, covunits=covunits,
-               confidence=confidence,
+               confidence=confidence, positiveCI=positiveCI,
                modelcall=modelcall, callstring=callstring)
 }
 
@@ -134,14 +143,16 @@ rhohat.lpp <- rhohat.lppm <-
            weights=NULL,
            method=c("ratio", "reweight", "transform"),
            horvitz=FALSE,
-           smoother=c("kernel", "local"),
+           smoother=c("kernel", "local", "decreasing", "increasing"),
            subset=NULL,
            nd=1000, eps=NULL, random=TRUE, 
            n=512, bw="nrd0", adjust=1, from=NULL, to=NULL, 
-           bwref=bw, covname, confidence=0.95) {
+           bwref=bw, covname, confidence=0.95, positiveCI) {
   callstring <- short.deparse(sys.call())
   smoother <- match.arg(smoother)
   method <- match.arg(method)
+  if(missing(positiveCI))
+    positiveCI <- (smoother == "local")
   if(missing(covname)) 
     covname <- sensiblevarname(short.deparse(substitute(covariate)), "X")
   if(is.null(adjust))
@@ -192,7 +203,7 @@ rhohat.lpp <- rhohat.lppm <-
                resolution=list(nd=nd, eps=eps, random=random),
                n=n, bw=bw, adjust=adjust, from=from, to=to,
                bwref=bwref, covname=covname, covunits=covunits,
-               confidence=confidence,
+               confidence=confidence, positiveCI=positiveCI,
                modelcall=modelcall, callstring=callstring)
 }
 
@@ -204,7 +215,7 @@ rhohatEngine <- function(model, covariate,
                          weights=NULL,
                          method=c("ratio", "reweight", "transform"),
                          horvitz=FALSE,
-                         smoother=c("kernel", "local"),
+                         smoother=c("kernel", "local", "decreasing", "increasing"),
                          resolution=list(), 
                          n=512, bw="nrd0", adjust=1, from=NULL, to=NULL, 
                          bwref=bw, covname, covunits=NULL, confidence=0.95,
@@ -217,16 +228,17 @@ rhohatEngine <- function(model, covariate,
                                subset=subset),
                           resolution))
   # unpack
-#  info   <- stuff$info
   values <- stuff$values
   # values at each data point
   ZX      <- values$ZX
   lambdaX <- values$lambdaX
   # values at each pixel
-  Zimage  <- values$Zimage
+  Zimage      <- values$Zimage
+  lambdaimage <- values$lambdaimage # could be multiple images
+  # values at each pixel (for .ppp, .ppm) or quadrature point (for .lpp, .lppm)
   Zvalues <- values$Zvalues
   lambda  <- values$lambda
-  ## weights
+    ## weights
   if(!is.null(weights)) {
     X <- data.ppm(model)
     if(is.im(weights)) 
@@ -241,9 +253,10 @@ rhohatEngine <- function(model, covariate,
   # normalising constants
   denom <- volume * (if(reference == "Lebesgue" || horvitz) 1 else mean(lambda))
   # info 
-  savestuff <- list(reference  = reference,
-                    horvitz    = horvitz,
-                    Zimage     = Zimage)
+  savestuff <- list(reference   = reference,
+                    horvitz     = horvitz,
+                    Zimage      = Zimage,
+                    lambdaimage = lambdaimage)
   # calculate rho-hat
   result <- rhohatCalc(ZX, Zvalues, lambda, denom,
                        ...,
@@ -293,19 +306,21 @@ rhohatCalc <- local({
   }
 
   rhohatCalc <- function(ZX, Zvalues, lambda, denom, ...,
-                         weights=NULL, lambdaX=NULL,
+                         weights=NULL, lambdaX,
                          method=c("ratio", "reweight", "transform"),
                          horvitz=FALSE, 
-                         smoother=c("kernel", "local"),
+                         smoother=c("kernel", "local", "decreasing", "increasing"),
                          n=512, bw="nrd0", adjust=1, from=NULL, to=NULL, 
                          bwref=bw, covname, confidence=0.95,
+                         positiveCI=(smoother == "local"),
+                         markovCI=TRUE,
                          covunits = NULL, modelcall=NULL, callstring=NULL,
                          savestuff=list()) {
     method <- match.arg(method)
     smoother <- match.arg(smoother)
     ## check availability of locfit package
     if(smoother == "local" && !requireNamespace("locfit", quietly=TRUE)) {
-      warning(paste("In", paste(dQuote(callstring), ":", sep=""),
+      warning(paste("In", paste0(dQuote(callstring), ":"),
                     "package", sQuote("locfit"), "is not available;",
                     "unable to perform local likelihood smoothing;",
                     "using kernel smoothing instead"),
@@ -332,14 +347,18 @@ rhohatCalc <- local({
     if(is.null(from)) from <- Zrange[1] 
     if(is.null(to))   to   <- Zrange[2]
     if(from > Zrange[1] || to < Zrange[2])
-      stop("Interval [from, to] = ", prange(c(from,to)), 
-           "does not contain the range of data values =", prange(Zrange))
+      stop(paste("In", paste0(dQuote(callstring), ":"),
+                 "interval [from, to] =", prange(c(from,to)), 
+                 "does not contain the range of data values =",
+                 prange(Zrange)),
+           call.=FALSE)
     ## critical constant for CI's
     crit <- qnorm((1+confidence)/2)
     percentage <- paste(round(100 * confidence), "%%", sep="")
     CIblurb <- paste("pointwise", percentage, "confidence interval")
-    ## estimate densities   
-    if(smoother == "kernel") {
+    ## estimate densities
+    switch(smoother,
+    kernel = {
       ## ............... kernel smoothing ......................
       ## reference density (normalised) for calculation
       ghat <- density(Zvalues,weights=if(horvitz) NULL else lambda/sum(lambda),
@@ -413,9 +432,22 @@ rhohatCalc <- local({
       vvvname <- "Variance of estimator"
       vvvlabel <- paste("bold(Var)~hat(%s)", paren(covname), sep="")
       sd <- sqrt(vvv)
-      hi <- yyy + crit * sd
-      lo <- yyy - crit * sd
-    } else {
+      if(!positiveCI) {
+        hi <- yyy + crit * sd
+        lo <- yyy - crit * sd
+      } else {
+        sdlog <- ifelse(yyy > 0, sd/yyy, 0)
+        sss <- exp(crit * sdlog)
+        hi <- yyy * sss
+        lo <- yyy / sss
+        if(markovCI) {
+          ## truncate extremely large confidence intervals
+          ## using Markov's Inequality
+          hi <- pmin(hi, yyy/(1-confidence))
+        }
+      }
+    },
+    local = {
       ## .................. local likelihood smoothing .......................
       xlim <- c(from, to)
       xxx <- seq(from, to, length=n)
@@ -463,35 +495,94 @@ rhohatCalc <- local({
              })
       vvvname <- "Variance of log of estimator"
       vvvlabel <- paste("bold(Var)~log(hat(%s)", paren(covname), ")", sep="")
-      sss <- exp(crit * sqrt(vvv))
-      hi <- yyy * sss
-      lo <- yyy / sss
-    }
+      sdlog <- sqrt(vvv)
+      if(positiveCI) {
+        sss <- exp(crit * sdlog)
+        hi <- yyy * sss
+        lo <- yyy / sss
+        if(markovCI) {
+          ## truncate extremely large confidence intervals
+          ## using Markov's Inequality
+          hi <- pmin(hi, yyy/(1-confidence))
+        }
+      } else {
+        hi <- yyy * (1 + crit * sdlog)
+        lo <- yyy * (1 - crit * sdlog)
+      }
+    },
+    increasing = ,
+    decreasing = {
+      ## .................. nonparametric maximum likelihood ............
+      if(is.null(weights)) weights <- rep(1, length(ZX))
+      #' observed (sorted)
+      oX <- order(ZX)
+      ZX <- ZX[oX]
+      weights <- weights[oX]
+      #' reference CDF
+      G <- ewcdf(Zvalues, lambda)
+      #' reference denominator ('area') at each observed value
+      if(smoother == "decreasing") {
+        areas <- denom * G(ZX)
+      } else {
+        areas <- denom * (1 - G(rev(ZX)))
+        weights <- rev(weights)
+      }
+      #' maximum upper sets algorithm
+      rho <- numeric(0)
+      darea <- diff(c(0, areas))
+      dcount <- weights
+      while(length(darea) > 0) {
+        u <- cumsum(dcount)/cumsum(darea)
+        if(any(bad <- !is.finite(u))) # divide by zero etc
+          u[bad] <- max(u[!bad], 0)
+        k <- which.max(u)
+        rho <- c(rho, rep(u[k], k))
+        darea <- darea[-(1:k)]
+        dcount <- dcount[-(1:k)]
+      }
+      rho <- c(rho, 0)
+      if(smoother == "increasing") rho <- rev(rho)
+      #' compute as a stepfun
+      rhofun <- stepfun(x = ZX, y=rho, right=TRUE, f=1)
+      #' evaluate on a grid
+      xlim <- c(from, to)
+      xxx <- seq(from, to, length=n)
+      yyy <- rhofun(xxx)
+      #'
+      vvv <- hi <- lo <- NULL
+      savestuff$rhofun <- rhofun
+    })
     ## pack into fv object
-    df <- data.frame(xxx=xxx, rho=yyy, var=vvv, hi=hi, lo=lo)
+    df <- data.frame(xxx=xxx, rho=yyy)
     names(df)[1] <- covname
-    desc <- c(paste("covariate", covname),
-              "Estimated intensity",
-              vvvname,
-              paste("Upper limit of", CIblurb),
-              paste("Lower limit of", CIblurb))
+    desc <- c(paste("covariate", covname), "Estimated intensity")
+    labl <- c(covname, paste("hat(%s)", paren(covname), sep=""))
+    if(did.variance <- !is.null(vvv)) {
+      df <- cbind(df, data.frame(var=vvv, hi=hi, lo=lo))
+      desc <- c(desc,
+                vvvname,
+                paste("Upper limit of", CIblurb),
+                paste("Lower limit of", CIblurb))
+      labl <- c(labl,
+                vvvlabel,
+                paste("%s[hi]", paren(covname), sep=""),
+                paste("%s[lo]", paren(covname), sep=""))
+    }
     rslt <- fv(df,
                argu=covname,
                ylab=substitute(rho(X), list(X=as.name(covname))),
                valu="rho",
                fmla= as.formula(paste(". ~ ", covname)),
                alim=range(ZX),
-               labl=c(covname,
-                 paste("hat(%s)", paren(covname), sep=""),
-                 vvvlabel,
-                 paste("%s[hi]", paren(covname), sep=""),
-                 paste("%s[lo]", paren(covname), sep="")),
+               labl=labl,
                desc=desc,
                unitname=covunits,
                fname="rho",
                yexp=substitute(rho(X), list(X=as.name(covname))))
-    fvnames(rslt, ".")  <- c("rho", "hi", "lo")
-    fvnames(rslt, ".s") <- c("hi", "lo")
+    if(did.variance) {
+      fvnames(rslt, ".")  <- c("rho", "hi", "lo")
+      fvnames(rslt, ".s") <- c("hi", "lo")
+    } else fvnames(rslt, ".")  <- "rho"
     ## pack up
     class(rslt) <- c("rhohat", class(rslt))
     ## add info
@@ -503,7 +594,9 @@ rhohatCalc <- local({
            ZX         = ZX,
            lambda     = lambda,
            method     = method,
-           smoother   = smoother)
+           smoother   = smoother,
+           confidence = confidence,
+           positiveCI = positiveCI)
     attr(rslt, "stuff") <- append(stuff, savestuff)
     return(rslt)
   }
@@ -525,7 +618,9 @@ print.rhohat <- function(x, ...) {
            splat("Function values are relative to fitted model")
            print(s$modelcall)
          })
+  NPMLE <- s$smoother %in% c("increasing", "decreasing")
   cat("Estimation method: ")
+  if(NPMLE) splat("nonparametric maximum likelihood") else
   switch(s$method,
          ratio={
            splat("ratio of fixed-bandwidth kernel smoothers")
@@ -545,13 +640,23 @@ print.rhohat <- function(x, ...) {
   switch(s$smoother,
          kernel={
            splat("Kernel density estimator")
-           splat("Actual smoothing bandwidth sigma = ",
+           splat("\tActual smoothing bandwidth sigma = ",
                  signif(s$sigma,5))
          },
-         local ={ splat("Local likelihood density estimator") }
+         local = splat("Local likelihood density estimator"),
+         increasing = splat("Increasing function of covariate"),
+         decreasing = splat("Decreasing function of covariate"),
+         splat("UNKNOWN")
          )
+  if(!NPMLE) {
+    positiveCI <- s$positiveCI %orifnull% (s$smoother == "local")
+    confidence <- s$confidence %orifnull% 0.95
+    splat("Pointwise", paste0(100 * confidence, "%"),
+          "confidence bands for rho(x)\n\t based on asymptotic variance of",
+          if(positiveCI) "log(rhohat(x))" else "rhohat(x)")
+  }
   splat("Call:", s$callstring)
-
+  cat("\n")
   NextMethod("print")
 }
 
@@ -560,9 +665,11 @@ plot.rhohat <- function(x, ..., do.rug=TRUE) {
   s <- attr(x, "stuff")
   covname <- s$covname
   asked.rug <- !missing(do.rug) && identical(rug, TRUE)
+  snam <- intersect(c("hi", "lo"), names(x))
+  if(length(snam) == 0) snam <- NULL
   out <- do.call(plot.fv,
                  resolve.defaults(list(x=x), list(...),
-                                  list(main=xname, shade=c("hi", "lo"))))
+                                  list(main=xname, shade=snam)))
   if(identical(list(...)$limitsonly, TRUE))
     return(out)
   if(do.rug) {
@@ -599,28 +706,44 @@ plot.rhohat <- function(x, ..., do.rug=TRUE) {
   invisible(NULL)
 }
 
-predict.rhohat <- function(object, ..., relative=FALSE,
-                    what=c("rho", "lo", "hi", "se")) {
-  trap.extra.arguments(..., .Context="in predict.rhohat")
-  what <- match.arg(what)
-  # extract info
-  s <- attr(object, "stuff")
-  reference <- s$reference
-  # convert to (linearly interpolated) function 
-  x <- with(object, .x)
-  y <- if(what == "se") sqrt(object[["var"]]) else object[[what]]
-  fun <- approxfun(x, y, rule=2)
-  # extract image of covariate
-  Z <- s$Zimage
-  # apply fun to Z
-  Y <- if(inherits(Z, "linim")) eval.linim(fun(Z)) else eval.im(fun(Z))
-  # adjust to reference baseline
-  if(reference != "Lebesgue" && !relative) {
-    lambda <- s$lambda
-    Y[] <- Y[] * lambda
+predict.rhohat <- local({
+
+  predict.rhohat <- function(object, ..., relative=FALSE,
+                             what=c("rho", "lo", "hi", "se")) {
+    trap.extra.arguments(..., .Context="in predict.rhohat")
+    what <- match.arg(what)
+    #' extract info
+    s <- attr(object, "stuff")
+    reference <- s$reference
+    #' check availability
+    if((what %in% c("lo", "hi", "se")) && !("hi" %in% names(object)))
+      stop("Standard error and confidence bands are not available in this object",
+           call.=FALSE)
+    #' convert to (linearly interpolated) function 
+    x <- with(object, .x)
+    y <- if(what == "se") sqrt(object[["var"]]) else object[[what]]
+    fun <- approxfun(x, y, rule=2)
+    #' extract image(s) of covariate
+    Z <- s$Zimage
+    #' apply fun to Z
+    Y <- if(is.im(Z)) evalfun(Z, fun) else solapply(Z, evalfun, f=fun)
+    #' adjust to reference baseline
+    if(reference != "Lebesgue" && !relative) {
+      Lam <- s$lambdaimage # could be 'im' or 'imlist'
+      Y <- Lam * Y 
+    }
+    return(Y)
   }
-  return(Y)
-}
+
+  evalfun <- function(X, f) {
+    force(f)
+    force(X)
+    if(is.linim(X)) eval.linim(f(X)) else 
+    if(is.im(X)) eval.im(f(X)) else NULL
+  }
+  
+  predict.rhohat
+})
 
 as.function.rhohat <- function(x, ..., value=".y", extrapolate=TRUE) {
   NextMethod("as.function")
@@ -629,7 +752,7 @@ as.function.rhohat <- function(x, ..., value=".y", extrapolate=TRUE) {
 simulate.rhohat <- function(object, nsim=1, ..., drop=TRUE) {
   trap.extra.arguments(..., .Context="in simulate.rhohat")
   lambda <- predict(object)
-  if(inherits(lambda, "linim")) {
+  if(is.linim(lambda) || (is.solist(lambda) && all(sapply(lambda, is.linim)))) {
     result <- rpoislpp(lambda, nsim=nsim, drop=drop)
   } else {
     result <- rpoispp(lambda, nsim=nsim, drop=drop)
