@@ -3,7 +3,7 @@
 #'
 #' Sparse 3D arrays represented as list(i,j,k,x)
 #' 
-#' $Revision: 1.25 $  $Date: 2017/07/13 02:01:19 $
+#' $Revision: 1.39 $  $Date: 2019/12/31 01:03:10 $
 #'
 
 sparse3Darray <- function(i=integer(0), j=integer(0), k=integer(0),
@@ -18,9 +18,9 @@ sparse3Darray <- function(i=integer(0), j=integer(0), k=integer(0),
               "are not yet supported by the Matrix package")
   stopifnot(length(dims) == 3)
   dims <- as.integer(dims)
-  if(!all(inside.range(i, c(1, dims[1])))) stop("indices i are outside range")
-  if(!all(inside.range(j, c(1, dims[2])))) stop("indices j are outside range")
-  if(!all(inside.range(k, c(1, dims[3])))) stop("indices k are outside range")
+  if(!all(i >= 1 & i <= dims[1])) stop("indices i are outside range")
+  if(!all(j >= 1 & j <= dims[2])) stop("indices j are outside range")
+  if(!all(k >= 1 & k <= dims[3])) stop("indices k are outside range")
   if(!is.null(dimnames)) {
     stopifnot(is.list(dimnames))
     stopifnot(length(dimnames) == 3)
@@ -99,7 +99,7 @@ as.sparse3Darray <- function(x, ...) {
       isnul <- sapply(dnlist, is.null)
       dnlist <- unique(dnlist[!isnul])
       if(length(dnlist) > 1) stop("Dimnames of matrices do not match")
-      dn <- if(length(dnlist) == 0) NULL else c(dn[[1L]], NULL)
+      dn <- if(length(dnlist) == 0) NULL else c(dnlist[[1L]], list(NULL))
       for(k in seq_len(n)) {
         mk <- as(x[[k]], "TsparseMatrix")
         kvalue <- if(length(mk@i) > 0) k else integer(0)
@@ -154,6 +154,8 @@ dimnames.sparse3Darray <- function(x) { x$dimnames }
 print.sparse3Darray <- function(x, ...) {
   dimx <- dim(x)
   cat("Sparse 3D array of dimensions", paste(dimx, collapse="x"), fill=TRUE)
+  if(prod(dimx) == 0)
+    return(invisible(NULL))
   dn <- dimnames(x) %orifnull% rep(list(NULL), 3)
   d3 <- dimx[3]
   dn3 <- dn[[3]] %orifnull% as.character(seq_len(d3))
@@ -162,7 +164,13 @@ print.sparse3Darray <- function(x, ...) {
   dim2 <- dimx[1:2]
   dn2 <- dn[1:2]
   if(typeof(x$x) == "complex") {
-    splat("\t[Complex-valued sparse matrices are not printable]")
+    splat("Complex-valued")
+    splat("\t\tReal component:")
+    stuff <- capture.output(eval(Re(x)))
+    cat(stuff[-1],sep="\n")
+    splat("\n\n\t\tImaginary component:")
+    stuff <- capture.output(eval(Im(x)))
+    cat(stuff[-1],sep="\n")
   } else {
     for(k in seq_along(pieces)) {
       cat(paste0("\n\t[ , , ", dn3[k], "]\n\n"))
@@ -212,15 +220,19 @@ as.array.sparse3Darray <- function(x, ...) {
       if(ncol(i) != 3)
         stop("If i is a matrix, it should have 3 columns", call.=FALSE)
       ## start with vector of 'zero' answers of the correct type
-      answer <- sparseVector(x=RelevantZero(x$x)[integer(0)],
+      answer <- sparseVector(x=RelevantEmpty(x$x),
                              i=integer(0),
                              length=nrow(i))
       ## values outside array return NA
-      if(any(bad <- !inside3Darray(dim(x), i)))
+      if(anybad <- !all(good <- inside3Darray(dim(x), i))) {
+        bad <- !good
         answer[bad] <- NA
+      }
       ## if entire array is zero, there is nothing to match
       if(length(x$x) == 0)
         return(answer)
+      ## restrict attention to entries inside array
+      igood <- if(anybad) i[good, , drop=FALSE] else i
       ## match desired indices to sparse entries
       varies <- (dimx > 1)
       nvary <- sum(varies)
@@ -232,7 +244,7 @@ as.array.sparse3Darray <- function(x, ...) {
         ## icode <- paste(i[,1], i[,2], i[,3], sep=",")
         ## dcode <- paste(x$i, x$j, x$k, sep=",")
 	## ------------------
-	m <- matchIntegerDataFrames(i, cbind(x$i, x$j, x$k))
+	mgood <- matchIntegerDataFrames(igood, cbind(x$i, x$j, x$k))
       } else if(nvary == 2) {
         ## effectively a sparse matrix
         ## ---- older code -----
@@ -241,25 +253,26 @@ as.array.sparse3Darray <- function(x, ...) {
         ## dcode <- paste(ijk[,varying[1]], ijk[,varying[2]], sep=",")
 	## ------------------
 	ijk <- cbind(x$i, x$j, x$k)
-	m <- matchIntegerDataFrames(i[,varying,drop=FALSE],
-	                            ijk[,varying,drop=FALSE])
+	mgood <- matchIntegerDataFrames(igood[,varying,drop=FALSE],
+                                        ijk[,varying,drop=FALSE])
       } else if(nvary == 1) {
         ## effectively a sparse vector
         ## ---- older code -----
         ## icode <- i[,varying]
         ## dcode <- switch(varying, x$i, x$j, x$k)
 	## ------------------
-	m <- match(i[,varying], switch(varying, x$i, x$j, x$k))
+	mgood <- match(igood[,varying], switch(varying, x$i, x$j, x$k))
       } else {
         ## effectively a single value
         ## ---- older code -----
         ## icode <- rep(1, nrow(i))
         ## dcode <- 1  # since we know length(x$x) > 0
-	m <- 1
+	mgood <- 1
       }
       ## insert any found elements
-      found <- !is.na(m)
-      answer[found] <- x$x[m[found]]
+      found <- logical(nrow(i))
+      found[good] <- foundgood <- !is.na(mgood)
+      answer[found] <- x$x[mgood[foundgood]]
       return(answer)
     }
     if(!(missing(i) && missing(j) && missing(k))) {
@@ -268,8 +281,18 @@ as.array.sparse3Darray <- function(x, ...) {
       K <- grokIndexVector(if(missing(k)) NULL else k, dimx[3], dn[[3]])
       IJK <- list(I,J,K)
       if(!all(sapply(lapply(IJK, getElement, name="full"), is.null))) {
-        ## indices exceed array bounds; result is a full array containing NA's
-        result <- as.array(x)[I$full$i, J$full$j, K$full$k, drop=drop]
+        ## some indices exceed array bounds;
+        ## result is a full array containing NA's
+        fullindices   <- lapply(IJK, fullIndexSequence)
+        strictindices <- lapply(IJK, strictIndexSequence)
+        result <- array(data=RelevantNA(x$x), dim=lengths(fullindices))
+        matches <- mapply(match, x=fullindices, table=strictindices)
+        ok <- lapply(lapply(matches, is.na), "!")
+        result[ok[[1]], ok[[2]], ok[[3]]] <-
+          as.array(x)[matches[[1]][ok[[1]]],
+                      matches[[2]][ok[[2]]],
+                      matches[[3]][ok[[3]]]]
+        if(drop) result <- result[,,,drop=TRUE]
         return(result)
       }
       IJK <- lapply(IJK, getElement, name="strict")
@@ -300,18 +323,21 @@ as.array.sparse3Darray <- function(x, ...) {
         outdf <- df
       } else {
         #' invert map to determine output positions (reorder/repeat entries)
-        imap <- Imap %orifnull% df$i
-        jmap <- Jmap %orifnull% df$j
-        kmap <- Kmap %orifnull% df$k
-        sn <- seq_len(nrow(df))
-        whichi <- split(seq_along(imap), factor(imap, levels=sn))
-        whichj <- split(seq_along(jmap), factor(jmap, levels=sn))
-        whichk <- split(seq_along(kmap), factor(kmap, levels=sn))
+        snI <- seq_len(I$n)
+        snJ <- seq_len(J$n)
+        snK <- seq_len(K$n)
+        imap <- Imap %orifnull% snI
+        jmap <- Jmap %orifnull% snJ
+        kmap <- Kmap %orifnull% snK
+        whichi <- split(seq_along(imap), factor(imap, levels=snI))
+        whichj <- split(seq_along(jmap), factor(jmap, levels=snJ))
+        whichk <- split(seq_along(kmap), factor(kmap, levels=snK))
         dat.i <- whichi[df$i]
         dat.j <- whichj[df$j]
         dat.k <- whichk[df$k]
         stuff <- mapply(expandwithdata,
-                        i=dat.i, j=dat.j, k=dat.k, x=df$x)
+                        i=dat.i, j=dat.j, k=dat.k, x=df$x,
+                        SIMPLIFY=FALSE)
         outdf <- rbindCompatibleDataFrames(stuff)
       }
       x <- sparse3Darray(i=outdf$i, j=outdf$j, k=outdf$k, x=outdf$x,
@@ -422,11 +448,16 @@ rbindCompatibleDataFrames <- function(x) {
   K <- grokIndexVector(if(missing(k)) NULL else k, dimx[3], dn[[3]])
   IJK <- list(I,J,K)
   if(!all(sapply(lapply(IJK, getElement, name="full"), is.null))) {
-    warning("indices exceed array bounds; using full array", call.=FALSE)
-    x <- as.array(x)
-    x[I$full$i, J$full$j, K$full$k] <- value
-    x <- as.sparse3Darray(x)
-    return(x)
+    warning("indices exceed array bounds; extending the array dimensions",
+            call.=FALSE)
+    fullindices   <- lapply(IJK, fullIndexSequence)
+    ## strictindices <- lapply(IJK, strictIndexSequence)
+    dnew <- pmax(dimx, sapply(fullindices, max))
+    result <- array(data=RelevantZero(x$x), dim=dnew)
+    result[cbind(x$i, x$j, x$k)] <- x$x
+    result[fullindices[[1]], fullindices[[2]], fullindices[[3]]] <- value
+    result <- as.sparse3Darray(result)
+    return(result)
   }
   IJK <- lapply(IJK, getElement, name="strict")
   if(all(sapply(IJK, getElement, name="nind") == 0)) {
@@ -517,8 +548,8 @@ rbindCompatibleDataFrames <- function(x) {
       pos1 <- replacementIndex(iv, IJK[[firstindex]])
       pos2 <- replacementIndex(seq_len(sd2), IJK[[secondindex]])
       xv   <- rep(xv, sd2)
-      pos1 <- rep(pos1, sd2)
       pos2 <- rep(pos2, each=length(pos1))
+      pos1 <- rep(pos1, sd2)
       pos3 <- if(length(pos1)) IJK[[which(collapsing)]]$i else integer(0)
       vdata <- data.frame(i=pos3, j=pos3, k=pos3, x=xv)
       vdata[,firstindex] <- pos1
@@ -606,6 +637,7 @@ anyNA.sparse3Darray <- function(x, recursive=FALSE) {
 RelevantZero <- function(x) vector(mode=typeof(x), length=1L)
 isRelevantZero <- function(x) identical(x, RelevantZero(x))
 RelevantEmpty <- function(x) vector(mode=typeof(x), length=0L)
+RelevantNA <- function(x) { RelevantZero(x)[2] }
 
 unionOfSparseIndices <- function(A, B) {
   #' A, B are data frames of indices i, j, k
@@ -795,15 +827,29 @@ Math.sparse3Darray <- function(x, ...){
   return(x)
 }
 
+Complex.sparse3Darray <- function(z) {
+  oo <- RelevantZero(z$x)
+  foo <- do.call(.Generic, list(z=oo))
+  if(!isRelevantZero(foo)) {
+    # result is a full array
+    result <- do.call(.Generic, list(z=as.array(z)))
+    return(result)
+  }
+  z$x <- do.call(.Generic, list(z=z$x))
+  return(z)
+}
+
 Summary.sparse3Darray <- function(..., na.rm=FALSE) {
   argh <- list(...)
   is3D <- sapply(argh, inherits, what="sparse3Darray")
   if(any(is3D)) {
-    xvalues <- lapply(argh[is3D], getElement, name="x")
-    argh[is3D] <- lapply(xvalues, .Generic, na.rm=na.rm)
+    xvalues   <- lapply(argh[is3D], getElement, name="x")
+    fullsizes <- sapply(lapply(argh[is3D], dim), prod)
+    argh[is3D] <- xvalues
+    #' zero entry should be appended if and only if there are any empty cells
     zeroes <- lapply(xvalues, RelevantZero)
-    fzeroes <- lapply(zeroes, .Generic, na.rm=na.rm)
-    argh <- append(argh, fzeroes)
+    zeroes <- zeroes[lengths(xvalues) < fullsizes]
+    argh <- append(argh, zeroes)
   }
   rslt <- do.call(.Generic, append(argh, list(na.rm=na.rm)))
   return(rslt)
