@@ -3,7 +3,7 @@
 #'
 #'  Utilities for sparse arrays
 #'
-#'  $Revision: 1.8 $  $Date: 2018/02/12 01:47:55 $
+#'  $Revision: 1.15 $  $Date: 2019/12/31 03:07:33 $
 #'
 
 #'  .............. completely generic ....................
@@ -99,7 +99,7 @@ expandSparse <- function(x, n, across) {
 }
 
 mapSparseEntries <- function(x, margin, values, conform=TRUE, across) {
-  # replace the NONZERO entries of sparse matrix or array
+  # replace the NONZERO entries of sparse vector, matrix or array
   # by values[l] where l is one of the slice indices
   dimx <- dim(x)
   if(is.null(dimx)) {
@@ -107,6 +107,20 @@ mapSparseEntries <- function(x, margin, values, conform=TRUE, across) {
     if(is.vector(x)) dimx <- length(x) else
     stop("Format of x is not understood", call.=FALSE)
   }
+  if(length(dimx) == 1) {
+    x <- as(x, "sparseVector")
+    i <- x@i
+    if(length(i) == 0) {
+      # no entries
+      return(x)
+    }
+    if(!missing(margin) && !is.null(margin)) stopifnot(margin == 1)
+    check.anySparseVector(values, dimx, things="entries", oneok=TRUE)
+    nv <- if(inherits(values, "sparseVector")) values@length else length(values)
+    yvalues <- if(nv > 1) as.vector(values[i]) else rep(values[1], length(i))
+    y <- sparseVector(i=i, x=yvalues, length=dimx)
+    return(y)
+  }
   if(inherits(x, "sparseMatrix")) {
     x <- as(x, Class="TsparseMatrix")
     if(length(x@i) == 0) {
@@ -115,14 +129,16 @@ mapSparseEntries <- function(x, margin, values, conform=TRUE, across) {
     }
     check.1.integer(margin)
     stopifnot(margin %in% 1:2)
-    check.nvector(values, dimx[margin], things=c("rows","columns")[margin],
-                  oneok=TRUE)
-    if(length(values) == 1) values <- rep(values, dimx[margin])
+    check.anySparseVector(values, dimx[margin],
+                          things=c("rows","columns")[margin],
+                          oneok=TRUE)
+    nv <- if(inherits(values, "sparseVector")) values@length else length(values)
     i <- x@i + 1L
     j <- x@j + 1L
     yindex <- switch(margin, i, j)
-    y <- sparseMatrix(i=i, j=j, x=values[yindex],
-                      dims=dimx, dimnames=dimnames(x))
+    yvalues <- if(nv > 1) values[yindex] else rep(values[1], length(yindex))
+    y <- sparseMatrix(i=i, j=j, x=yvalues, dims=dimx, dimnames=dimnames(x))
+    y <- drop0(y)
     return(y)
   }
   if(inherits(x, "sparse3Darray")) {
@@ -149,20 +165,21 @@ mapSparseEntries <- function(x, margin, values, conform=TRUE, across) {
       ijk <- apply(ijk, 2, rep, times=nslice)
       ijk[, across] <- rep(seq_len(nslice), each=npattern)
     }
-    if(is.vector(values)) {
+    if(is.vector(values) || inherits(values, "sparseVector")) {
       # vector of values matching margin extent
-      check.nvector(values, dimx[margin],
-                    things=c("rows","columns","planes")[margin],
-                    oneok=TRUE)
-      if(length(values) == 1) values <- rep(values, dimx[margin])
+      check.anySparseVector(values, dimx[margin],
+                            things=c("rows","columns","planes")[margin],
+                            oneok=TRUE)
+      nv <- if(inherits(values, "sparseVector")) values@length else length(values)
       yindex <- ijk[,margin]
+      yvalues <- if(nv > 1) values[yindex] else rep(values[1], length(yindex))
       y <- sparse3Darray(i=ijk[,1],
                          j=ijk[,2],
                          k=ijk[,3],
-                         x=values[yindex],
+                         x=yvalues,
                          dims=dimx, dimnames=dimnames(x))
       return(y)
-    } else if(is.matrix(values)) {
+    } else if(is.matrix(values) || inherits(values, "sparseMatrix")) {
       #' matrix of values.
       force(across)
       stopifnot(across != margin) 
@@ -196,7 +213,7 @@ applySparseEntries <- local({
     ## apply vectorised function 'f' only to the nonzero entries of 'x'
     if(inherits(x, "sparseMatrix")) {
       x <- applytoxslot(x, f, ...)
-    } else if(inherits(x, "sparse3Dmatrix")) {
+    } else if(inherits(x, "sparse3Darray")) {
       x <- applytoxentry(x, f, ...)
     } else {
       x <- f(x, ...)
@@ -231,3 +248,33 @@ applySparseEntries <- local({
   applySparseEntries
 })
 
+check.anySparseVector <- function(v, npoints=NULL, fatal=TRUE, things="data points",
+                                  naok=FALSE, warn=FALSE, vname, oneok=FALSE) {
+  # vector, factor or sparse vector of values for each point/thing
+  if(missing(vname))
+    vname <- sQuote(deparse(substitute(v)))
+  whinge <- NULL
+  isVector <- is.atomic(v) && is.null(dim(v))
+  isSparse <- inherits(v, "sparseVector")
+  nv <- if(isSparse) v@length else length(v)
+  if(!isVector && !isSparse) 
+    whinge <- paste(vname, "is not a vector, factor or sparse vector")
+  else if(!(is.null(npoints) || (nv == npoints)) &&
+          !(oneok && nv == 1)) 
+    whinge <- paste("The length of", vname,
+                    paren(paste0("=", nv)), 
+                    "should equal the number of", things,
+                    paren(paste0("=", npoints)))
+  else if(!naok && anyNA(v))
+    whinge <- paste("Some values of", vname, "are NA or NaN")
+  #
+  if(!is.null(whinge)) {
+    if(fatal) stop(whinge)
+    if(warn) warning(whinge)
+    ans <- FALSE
+    attr(ans, "whinge") <- whinge
+    return(ans)
+  }
+  return(TRUE)
+}
+
