@@ -3,7 +3,7 @@
 #
 #    summary() method for class "ppm"
 #
-#    $Revision: 1.67 $   $Date: 2014/04/05 08:06:25 $
+#    $Revision: 1.78 $   $Date: 2019/10/11 06:23:19 $
 #
 #    summary.ppm()
 #    print.summary.ppm()
@@ -26,12 +26,13 @@ summary.ppm <- local({
     return(ar[ar != "..."])
   }
 
-  summary.ppm <- function(object, ..., quick=FALSE) {
+  summary.ppm <- function(object, ..., quick=FALSE, fine=FALSE) {
     verifyclass(object, "ppm")
 
     x <- object
     y <- list()
-  
+    class(y) <- "summary.ppm"
+
     #######  Extract main data components #########################
 
     QUAD <- object$Q
@@ -50,7 +51,7 @@ summary.ppm <- local({
     virgin <- min(package_version(c(mpl.ver, int.ver)))
     
     y$antiquated <- antiquated <- (virgin <= package_version("1.5"))
-    y$old        <- old        <- (virgin < majorminorversion(current))
+    y$old        <- (virgin < majorminorversion(current))
 
     y$version    <- as.character(virgin)
     
@@ -83,7 +84,9 @@ summary.ppm <- local({
     ######  Fitting algorithm ########################################
 
     y$method <- x$method
-  
+
+    y$VB <- x$internal$VB
+    
     y$problems <- x$problems
 
     y$fitter <- if(!is.null(x$fitter)) x$fitter else "unknown"
@@ -92,9 +95,11 @@ summary.ppm <- local({
 
     ######  Coefficients were changed after fit? #####################
   
-    y$projected <- identical(x$projected, TRUE)
-    y$changedcoef <- y$projected || !is.null(x$coef.orig)
+    y$projected <- yproj <- identical(x$projected, TRUE)
+    y$changedcoef <- yproj || !is.null(x$coef.orig)
 
+    y$valid <- valid.ppm(x, warn=FALSE)
+      
     ######  Extract fitted model coefficients #########################
 
     y$entries$coef <- COEFS <- x$coef
@@ -110,10 +115,8 @@ summary.ppm <- local({
 
     # Exit here if quick=TRUE
     
-    if(identical(quick, TRUE)) {
-      class(y) <- "summary.ppm"
+    if(identical(quick, TRUE)) 
       return(y)
-    }
 
     ######  Does it have external covariates?  ####################
 
@@ -131,6 +134,7 @@ summary.ppm <- local({
                      covfunargs    = NULL,
                      has.xargs     = FALSE,
                      xargmap       = NULL))
+    class(y) <- "summary.ppm"
 
     if(!antiquated) {
       covars <- x$covariates
@@ -162,7 +166,7 @@ summary.ppm <- local({
           y$covar.descrip[isfun] <- unlist(lapply(funs, fdescrip))
           # find any extra arguments (after args 1 & 2) explicitly named
           fargs <- lapply(funs, xargs)
-          nxargs <- unlist(lapply(fargs, length))
+          nxargs <- lengths(fargs)
           y$has.xargs <- any(nxargs > 0)
           if(y$has.xargs) {
             # identify which function arguments are fixed in the call
@@ -217,7 +221,7 @@ summary.ppm <- local({
 
     if(y$poisson && y$no.trend) {
       # uniform Poisson process
-      y$trend$value <- lambda <- exp(COEFS[[1]])
+      y$trend$value <- exp(COEFS[[1]])
       y$trend$label <- switch(y$marktype,
                               unmarked="Uniform intensity",
                               multitype="Uniform intensity for each mark level",
@@ -256,7 +260,7 @@ summary.ppm <- local({
         trendbits <- COEFS
       else {
         agree <- outer(names(COEFS), Vnames, "==")
-        whichbits <- apply(!agree, 1, all)
+        whichbits <- matrowall(!agree)
         trendbits <- COEFS[whichbits]
       }
       y$trend$label <- ngettext(length(trendbits),
@@ -270,11 +274,15 @@ summary.ppm <- local({
     if(is.character(quick) && (quick == "no variances"))
       return(y)
 
+    # Exit before SE for variational Bayes
+    if(!is.null(x$internal$VB))
+      return(y)
+    
     if(length(COEFS) > 0) {
       # compute standard errors
       se <- x$internal$se
       if(is.null(se)) {
-        vc <- vcov(x, matrix.action="warn")
+        vc <- vcov(x, fine=fine, matrix.action="warn", invalid.action="silent")
         if(!is.null(vc)) {
           se <- if(is.matrix(vc)) sqrt(diag(vc)) else
                 if(length(vc) == 1) sqrt(vc) else NULL
@@ -284,19 +292,19 @@ summary.ppm <- local({
         two <- qnorm(0.975)
         lo <- COEFS - two * se
         hi <- COEFS + two * se
-        pval <- 2 * pnorm(abs(COEFS)/se, lower.tail=FALSE)
-        psig <- cut(pval, c(0,0.001, 0.01, 0.05, 1, Inf),
-                    labels=c("***", "**", "*", "  ", "na"),
+        zval <- COEFS/se
+        pval <- 2 * pnorm(abs(zval), lower.tail=FALSE)
+        psig <- cut(pval, c(0,0.001, 0.01, 0.05, 1),
+                    labels=c("***", "**", "*", "  "),
                     include.lowest=TRUE)
-        notapplic <- names(COEFS) %in% c("(Intercept)", "log(lambda)")
-        psig[notapplic] <- "na"
         # table of coefficient estimates with SE and 95% CI
-        y$coefs.SE.CI <- data.frame(Estimate=COEFS, S.E.=se, Ztest=psig,
-                                    CI95.lo=lo, CI95.hi=hi)
+        y$coefs.SE.CI <- data.frame(Estimate=COEFS, S.E.=se,
+                                    CI95.lo=lo, CI95.hi=hi,
+                                    Ztest=psig,
+                                    Zval=zval)
       }
     }
   
-    class(y) <- "summary.ppm"
     return(y)
   }
   
@@ -315,12 +323,12 @@ print.summary.ppm <- function(x, ...) {
   
   if(is.null(x$args)) {
     # this is the quick version
-    cat(paste(x$name, "\n"))
+    splat(x$name)
     return(invisible(NULL))
   }
 
   # otherwise - full details
-  cat("Point process model\n")
+  splat("Point process model")
   fitter <- if(!is.null(x$fitter)) x$fitter else "unknown"
   methodchosen <-
     if(is.null(x$method))
@@ -329,66 +337,77 @@ print.summary.ppm <- function(x, ...) {
       switch(x$method,
              mpl={
                if(x$poisson) {
-                 # Poisson process
-                 "maximum likelihood (Berman-Turner approximation)"
+                 # Poisson process 
+                "maximum likelihood (Berman-Turner approximation)"
                } else {
                  "maximum pseudolikelihood (Berman-Turner approximation)"
                } 
              },
              logi={
-               if(!x$poisson) {
-                 "maximum pseudolikelihood (logistic regression approximation)"
+               if(is.null(x$VB)){
+                 if(x$poisson) {
+                   # Poisson process
+                   "maximum likelihood (logistic regression approximation)"
+                 } else {
+                   "maximum pseudolikelihood (logistic regression approximation)"
+                 }
                } else {
-                 # Poisson process
-                 "maximum likelihood (logistic regression approximation)"
-               } 
+                 "maximum posterior density (variational Bayes approximation)"
+               }
              },
              ho="Huang-Ogata method (approximate maximum likelihood)",
              paste("unrecognised method", sQuote(x$method)))
-  cat(paste("Fitting method:", methodchosen, "\n"))
+  splat("Fitting method:", methodchosen)
   howfitted <- switch(fitter,
                       exact= "analytically",
                       gam  = "using gam()",
                       glm  = "using glm()",
                       ho   = NULL,
                       paste("using unrecognised fitter", sQuote(fitter)))
-  if(!is.null(howfitted)) cat(paste("Model was fitted", howfitted, "\n"))
+  if(!is.null(howfitted)) splat("Model was fitted", howfitted)
   if(fitter %in% c("glm", "gam")) {
-    if(x$converged) cat("Algorithm converged\n")
-    else cat("*** Algorithm did not converge ***\n")
+    if(x$converged) splat("Algorithm converged")
+    else splat("*** Algorithm did not converge ***")
   }
   if(x$projected)
-    cat("Fit was projected to obtain a valid point process model\n")
+    splat("Fit was projected to obtain a valid point process model")
 
   cat("Call:\n")
   print(x$args$call)
 
   if(x$old) 
-    cat(paste("** Executed by old spatstat version", x$version, " **\n"))
+    splat("** Executed by old spatstat version", x$version, " **")
   
-  cat(paste("Edge correction:", dQuote(x$args$correction), "\n"))
+  splat("Edge correction:", dQuote(x$args$correction))
   if(x$args$correction == "border")
-    cat(paste("\t[border correction distance r =", x$args$rbord,"]\n"))
-
-  cat("\n----------------------------------------------------\n")
+    splat("\t[border correction distance r =", x$args$rbord,"]")
 
   # print summary of quadrature scheme
+  if(is.null(x$quad))
+    return(invisible(NULL))
+  ruletextline()
   print(x$quad)
-  
-  cat("\n----------------------------------------------------\n")
-  cat("FITTED MODEL:\n\n")
+
+
+  ## start printing trend information
+  if(is.null(x$no.trend))
+    return(invisible(NULL))
+
+  ruletextline()
+  splat("FITTED MODEL:")
+  parbreak()
 
   # This bit is currently identical to print.ppm()
   # except for a bit more fanfare
   # and the inclusion of the 'gory details' bit
   
   notrend <-    x$no.trend
-  stationary <- x$stationary
+#  stationary <- x$stationary
   poisson <-    x$poisson
   markeddata <- x$marked
   multitype  <- x$multitype
         
-  markedpoisson <- poisson && markeddata
+#  markedpoisson <- poisson && markeddata
 
   # ----------- Print model type -------------------
         
@@ -397,37 +416,43 @@ print.summary.ppm <- function(x, ...) {
 
   if(markeddata) mrk <- x$entries$marks
   if(multitype) {
-    cat("Possible marks: \n")
+    splat("Possible marks:")
     cat(paste(levels(mrk)))
   }
 
   # ----- trend --------------------------
 
-  cat(paste("\n\n ---- ", x$trend$name, ": ----\n\n", sep=""))
+  if(length(x$trend) == 0)
+    return(invisible(NULL))
+  
+  parbreak()
+  splat(paste0("---- ", x$trend$name, ": ----"))
+  parbreak()
 
   if(!notrend) {
-    cat("Trend formula: ")
-    print(x$trend$formula)
+    splat("Log",
+          if(poisson) "intensity:" else "trend:",
+          pasteFormula(x$trend$formula))
     if(x$uses.covars) 
-      cat(paste("Model depends on external",
-                ngettext(length(x$covars.used), "covariate", "covariates"),
-                commasep(sQuote(x$covars.used)), "\n"))
+      splat("Model depends on external",
+            ngettext(length(x$covars.used), "covariate", "covariates"),
+            commasep(sQuote(x$covars.used)))
   }
   if(x$has.covars) {
     if(notrend || !x$uses.covars)
-      cat("Model object contains external covariates\n")
+      splat("Model object contains external covariates")
     isdf <- identical(x$covars.are.df, TRUE)
     if(!is.null(cd <- x$covar.descrip)) {
       # print description of each covariate
-      cat(paste("\nCovariates provided",
-                if(isdf) " (in data frame)" else NULL,
-                ":\n", sep=""))
+      splat(paste0("Covariates provided",
+                   if(isdf) " (in data frame)" else NULL,
+                   ":"))
       namescd <- names(cd)
       for(i in seq_along(cd))
-        cat(paste("\t", namescd[i], ": ", cd[i], "\n", sep=""))
+        splat(paste0("\t", namescd[i], ": ", cd[i]))
     }
     if(!is.null(cfa <- x$covfunargs) && length(cfa) > 0) {
-      cat("Covariate function arguments (covfunargs) provided:\n")
+      splat("Covariate function arguments (covfunargs) provided:")
       namescfa <- names(cfa)
       for(i in seq_along(cfa)) {
         cat(paste(namescfa[i], "= "))
@@ -439,7 +464,8 @@ print.summary.ppm <- function(x, ...) {
     }
   }
 
-  cat(paste("\n", x$trend$label, ":\n", sep=""))
+  parbreak()
+  splat(paste0(x$trend$label, ":"))
   
   tv <- x$trend$value
   if(!is.list(tv))
@@ -456,19 +482,25 @@ print.summary.ppm <- function(x, ...) {
   
   # ---- Interaction ----------------------------
 
+  
   if(!poisson) {
-    cat("\n\n ---- Interaction: -----\n\n")
+    parbreak()
+    splat(" ---- Interaction: -----")
+    parbreak()
     print(x$interaction)
   }
 
   ####### Gory details ###################################
-  cat("\n\n----------- gory details -----\n")
+  parbreak()
+  splat("----------- gory details -----")
+  parbreak()
   COEFS <- x$entries$coef
   
-  cat("\nFitted regular parameters (theta): \n")
+  splat("Fitted regular parameters (theta):")
   print(COEFS)
 
-  cat("\nFitted exp(theta): \n")
+  parbreak()
+  splat("Fitted exp(theta):")
   print(exp(unlist(COEFS)))
 
   ##### Warnings issued #######
@@ -480,7 +512,21 @@ print.summary.ppm <- function(x, ...) {
              if(is.list(a) && !is.null(p <- a$print))
                cat(paste("Problem:\n", p, "\n\n"))
            })
-          
+
+  vali <- x$valid
+  if(identical(vali, FALSE) && waxlyrical("errors")) {
+    parbreak()
+    splat("*** Model is not valid ***")
+    if(!all(is.finite(x$entries$coef))) {
+      splat("*** Some coefficients are NA or Inf ***")
+    } else {
+      splat("*** Interaction parameters are outside valid range ***")
+    }
+  } else if(is.na(vali) && waxlyrical("extras")) {
+    parbreak()
+    splat("[Validity of model could not be checked]")
+  }
+  
   return(invisible(NULL))
 }
 
