@@ -1,7 +1,7 @@
 ##
 ## symbolmap.R
 ##
-##   $Revision: 1.35 $  $Date: 2017/08/31 08:48:24 $
+##   $Revision: 1.37 $  $Date: 2020/01/11 04:45:05 $
 ##
 
 symbolmap <- local({
@@ -144,6 +144,15 @@ symbolmap <- local({
 
 symbolmaptype <- function(x) { attr(x, "stuff")$type }
 
+symbolmapdomain <- function(x) {
+  stuff <- attr(x, "stuff")
+  d <- switch(stuff$type,
+              constant = { integer(0) },
+              discrete = { stuff$inputs },
+              continuous = { stuff$range })
+  return(d)
+}
+
 update.symbolmap <- function(object, ...) {
   y <- attr(object, "stuff")
   oldargs <- append(y[["parlist"]], y[c("inputs", "range")])
@@ -600,13 +609,14 @@ plan.legend.layout <- function(B,
     stopifnot(sep > 0)
   }
   if(is.null(map) || !inherits(map, "symbolmap")) {
+    vv <- NULL
     textlength <- 8
   } else {
     vv <- with(attr(map, "stuff"),
                if(type == "discrete") inputs else prettyinside(range))
     textlength <- max(nchar(paste(vv)))
   }
-  if(started) {
+  if(started && !is.null(vv)) {
     textwidth <- max(strwidth(vv))
     textheight <- max(strheight(vv))
   } else {
