@@ -3,7 +3,7 @@
 #
 #	A class 'owin' to define the "observation window"
 #
-#	$Revision: 4.176 $	$Date: 2017/09/11 16:35:38 $
+#	$Revision: 4.189 $	$Date: 2019/11/11 10:15:17 $
 #
 #
 #	A window may be either
@@ -80,7 +80,7 @@ owin <- local({
   if(nargs() == 1 && !missing(xrange) && is.owin(xrange))
     return(xrange)
   
-  unitname <- as.units(unitname)
+  unitname <- as.unitname(unitname)
 
   ## Exterminate ambiguities
   poly.given <- !is.null(poly)
@@ -93,7 +93,11 @@ owin <- local({
      
   if(missing(xrange) != missing(yrange))
     stop("If one of xrange, yrange is specified then both must be.")
-
+  if(!missing(xrange)) {
+    xrange <- unname(xrange)
+    yrange <- unname(yrange)
+  }
+    
   # convert data frames to vanilla lists
   if(poly.given) {
     if(is.data.frame(poly))
@@ -156,7 +160,7 @@ owin <- local({
     
     if(psingle) {
       # single boundary polygon
-      bdry <- list(poly)
+      bdry <- unname(list(poly))
       if(check || calculate) {
         w.area <- Area.xypolygon(poly)
         if(w.area < 0)
@@ -165,9 +169,9 @@ owin <- local({
       }
     } else {
       # multiple boundary polygons
-      bdry <- poly
+      bdry <- unname(poly)
       if(check || calculate) {
-        w.area <- unlist(lapply(poly, Area.xypolygon))
+        w.area <- sapply(poly, Area.xypolygon)
         if(sum(w.area) < 0)
           stop(paste("Area of window is negative;\n",
                      "check that all polygons were traversed",
@@ -179,7 +183,7 @@ owin <- local({
     if(missing(xrange))
       xrange <- actual.xrange
     else if(check) {
-      if(!is.vector(xrange) || length(xrange) != 2 || xrange[2L] <= xrange[1L])
+      if(!is.vector(xrange) || length(xrange) != 2 || xrange[2L] < xrange[1L])
         stop("xrange should be a vector of length 2 giving (xmin, xmax)")
       if(!all(xrange == range(c(xrange, actual.xrange))))
         stop("polygon's x coordinates outside xrange")
@@ -189,7 +193,7 @@ owin <- local({
     if(missing(yrange))
       yrange <- actual.yrange
     else if(check) {
-      if(!is.vector(yrange) || length(yrange) != 2 || yrange[2L] <= yrange[1L])
+      if(!is.vector(yrange) || length(yrange) != 2 || yrange[2L] < yrange[1L])
         stop("yrange should be a vector of length 2 giving (ymin, ymax)")
       if(!all(yrange == range(c(yrange, actual.yrange))))
       stop("polygon's y coordinates outside yrange")
@@ -292,9 +296,9 @@ owin <- local({
         xrange <- c(0,nc)
         yrange <- c(0,nr)
       } else if(check) {
-        if(!is.vector(xrange) || length(xrange) != 2 || xrange[2L] <= xrange[1L])
+        if(!is.vector(xrange) || length(xrange) != 2 || xrange[2L] < xrange[1L])
           stop("xrange should be a vector of length 2 giving (xmin, xmax)")
-        if(!is.vector(yrange) || length(yrange) != 2 || yrange[2L] <= yrange[1L])
+        if(!is.vector(yrange) || length(yrange) != 2 || yrange[2L] < yrange[1L])
           stop("yrange should be a vector of length 2 giving (ymin, ymax)")
       }
       xstep <- diff(xrange)/nc
@@ -304,14 +308,14 @@ owin <- local({
     }
 
     out <- list(type     = "mask",
-                xrange   = xrange,
-                yrange   = yrange,
+                xrange   = unname(xrange),
+                yrange   = unname(yrange),
                 dim      = c(nr, nc),
-                xstep    = xstep,
-                ystep    = ystep,
+                xstep    = unname(xstep),
+                ystep    = unname(ystep),
                 warnings = .Spatstat.Image.Warning,
-                xcol    = xcol, 
-                yrow    = yrow,
+                xcol    = unname(xcol), 
+                yrow    = unname(yrow),
                 m       = mask,
                 units   = unitname)
     class(out) <- "owin"
@@ -415,8 +419,8 @@ as.owin.data.frame <- function(W, ..., step, fatal=TRUE) {
   iz <- mch[3L]
   df <- data.frame(x=W[,ix], y=W[,iy], z=as.logical(W[,iz]))
   with(df, {
-    xx <- sort(unique(x))
-    yy <- sort(unique(y))
+    xx <- sortunique(x)
+    yy <- sortunique(y)
     jj <- match(x, xx)
     ii <- match(y, yy)
     ## make logical matrix (for incomplete x, y sequence)
@@ -457,8 +461,9 @@ as.owin.default <- function(W, ..., fatal=TRUE) {
   ## Tries to interpret data as an object of class 'owin'
   ## W may be
   ##	a structure with entries xrange, yrange
-  ##	a four-element vector (interpreted xmin, xmax, ymin, ymax)
   ##	a structure with entries xl, xu, yl, yu
+  ##	a structure with entries xmin, xmax, ymin, ymax
+  ##	a four-element vector (interpreted xmin, xmax, ymin, ymax)
   ##	an object with attribute "bbox"
 
   if(inherits(W, "box3")) {
@@ -471,8 +476,9 @@ as.owin.default <- function(W, ..., fatal=TRUE) {
   if(checkfields(W, c("xrange", "yrange"))) {
     Z <- owin(W$xrange, W$yrange)
     return(Z)
-  } else if(is.vector(W) && is.numeric(W) && length(W) == 4) {
-    Z <- owin(W[1:2], W[3:4])
+  } else if(checkfields(W, c("xmin", "xmax", "ymin", "ymax"))) {
+    W <- as.list(W)
+    Z <- owin(c(W$xmin, W$xmax),c(W$ymin, W$ymax))
     return(Z)
   } else if(checkfields(W, c("xl", "xu", "yl", "yu"))) {
     W <- as.list(W)
@@ -483,11 +489,21 @@ as.owin.default <- function(W, ..., fatal=TRUE) {
     V <- as.list(W$area)
     Z <- owin(c(V$xl, V$xu),c(V$yl, V$yu))
     return(Z)
+  } else if(is.vector(W) && is.numeric(W) && length(W) == 4) {
+    Z <- owin(W[1:2], W[3:4])
+    return(Z)
   } else if(!is.null(Z <- attr(W, "bbox"))) {
     return(as.owin(Z, ..., fatal=fatal))
-  } else if(fatal)
-    stop("Can't interpret W as a window")
-  else return(NULL)
+  } else if(any(c("SpatialPolygons", "SpatialPolygonsDataFrame")
+                %in% class(W))) {
+    gripe <- "The package 'maptools' is needed to convert this data type"
+    if(fatal) stop(gripe, call.=FALSE) else warning(gripe, call.=FALSE)
+    return(NULL)
+  } else {
+    #' no idea
+    if(fatal) stop("Can't interpret W as a window", call.=FALSE)
+    return(NULL)
+  }
 }		
 
 #
@@ -500,6 +516,11 @@ Frame <- function(X) { UseMethod("Frame") }
 
 Frame.default <- function(X) { as.rectangle(X) }
 
+"Frame<-.default" <- function(X, value) {
+  Frame(Window(X)) <- value
+  return(X)
+}
+
 ## .........................................................
 
 as.rectangle <- function(w, ...) {
@@ -523,7 +544,7 @@ as.mask <- function(w, eps=NULL, dimyx=NULL, xy=NULL) {
 #	dimyx:		   dimensions of pixel raster
 #       xy:                coordinates of pixel raster
   nonamedargs <- is.null(eps) && is.null(dimyx) && is.null(xy)
-  uname <- as.units(NULL)
+  uname <- as.unitname(NULL)
   if(!missing(w) && !is.null(w)) {
     if(is.data.frame(w)) return(owin(mask=w, xy=xy))
     if(is.matrix(w)) {
@@ -600,8 +621,8 @@ as.mask <- function(w, eps=NULL, dimyx=NULL, xy=NULL) {
       if(!checkfields(xy, c("x", "y")))
         stop(paste(sQuote("xy"),
                    "should be a list containing two vectors x and y"))
-      x <- sort(unique(xy$x))
-      y <- sort(unique(xy$y))
+      x <- sortunique(xy$x)
+      y <- sortunique(xy$y)
       # derive other parameters
       nr <- length(y)
       nc <- length(x)
@@ -928,8 +949,10 @@ inside.owin <- function(x, y, w) {
   # test whether (x,y) is inside window w
   # x, y may be vectors 
 
-  if(missing(y) && all(c("x", "y") %in% names(x)))
-    return(inside.owin(x$x, x$y, w))
+  if((missing(y) || is.null(y)) && all(c("x", "y") %in% names(x))) {
+    y <- x$y
+    x <- x$x
+  }
 
   w <- as.owin(w)
 
@@ -956,8 +979,8 @@ inside.owin <- function(x, y, w) {
            framesize <- max(diff(xr), diff(yr))
            if(framesize > 1e6 || framesize < 1e-6) {
              ## rescale to avoid numerical overflow
-             scalefac <- framesize/100
-             w <- rescale(w, scalefac)
+             scalefac <- as.numeric(framesize)/100
+             w <- as.polygonal(rescale(w, scalefac))
              x <- x/scalefac
              y <- y/scalefac
            }
@@ -1011,9 +1034,9 @@ print.owin <- function(x, ..., prefix="window: ") {
            rectname <- paste0(prefix, "rectangle =")
          },
          polygonal={
-           splat(paste0(prefix, "polygonal boundary"))
-           if(length(x$bdry) == 0)
-             splat("window is empty")
+           nonemp <- (length(x$bdry) != 0)
+           splat(paste0(prefix,
+                        if(nonemp) "polygonal boundary" else "empty"))
            rectname <- "enclosing rectangle:"
          },
          mask={
@@ -1116,6 +1139,13 @@ print.summary.owin <- function(x, ...) {
         "x",
         prange(zapsmall(x$yrange)),
         pluralunits)
+  if(x$xrange[1] != 0 || x$yrange[1] != 0) {
+    width <- diff(x$xrange)
+    height <- diff(x$yrange)
+    blank <- paste(rep(" ", nchar(rectname)), collapse="") 
+    splat(blank, paren(paste(signif(width, 4), "x", signif(height, 4),
+                             pluralunits)))
+  }
   Area <- signif(x$area, 6)
   splat("Window area =", Area, "square",
         if(Area == 1) singularunits else pluralunits)
