@@ -1,7 +1,7 @@
 #
 #	wingeom.R	Various geometrical computations in windows
 #
-#	$Revision: 4.123 $	$Date: 2017/09/04 05:10:45 $
+#	$Revision: 4.135 $	$Date: 2020/02/06 06:39:51 $
 #
 
 volume.owin <- function(x) { area.owin(x) }
@@ -101,9 +101,9 @@ eroded.areas <- function(w, r, subset=NULL) {
 }	
 
 even.breaks.owin <- function(w) {
-	verifyclass(w, "owin")
-        Rmax <- diameter(w)
-        make.even.breaks(Rmax, Rmax/(100 * sqrt(2)))
+  verifyclass(w, "owin")
+  Rmax <- diameter(w)
+  make.even.breaks(bmax=Rmax, npos=128)
 }
 
 unit.square <- function() { owin(c(0,1),c(0,1)) }
@@ -192,7 +192,7 @@ overlap.owin <- function(A, B) {
   # compute the area of overlap between two windows
   
   # check units
-  if(!compatible.units(unitname(A), unitname(B)))
+  if(!compatible(unitname(A), unitname(B)))
     warning("The two windows have incompatible units of length")
   
   At <- A$type
@@ -262,7 +262,7 @@ function(x, i, ...) {
 #
 #
 
-intersect.owin <- function(..., fatal=TRUE, p) {
+intersect.owin <- function(..., fatal=FALSE, p) {
   argh <- list(...)
   ## p is a list of arguments to polyclip::polyclip
   if(missing(p) || is.null(p)) p <- list()
@@ -279,7 +279,7 @@ intersect.owin <- function(..., fatal=TRUE, p) {
     }
   }
   ## look for window arguments
-  isowin <- sapply(argh, is.owin)
+  isowin <- as.logical(sapply(argh, is.owin))
   if(any(!isowin))
     warning("Some arguments were not windows")
   argh <- argh[isowin]
@@ -292,9 +292,18 @@ intersect.owin <- function(..., fatal=TRUE, p) {
 
   ## at least one window
   A <- argh[[1L]]
+  if(is.empty(A)) {
+    if(fatal) stop("Intersection is empty", call.=FALSE)
+    return(A)
+  }
   if(nwin == 1) return(A)
+
   ## at least two windows
   B <- argh[[2L]]
+  if(is.empty(B)) {
+    if(fatal) stop("Intersection is empty", call.=FALSE)
+    return(B)
+  }
   
   if(nwin > 2) {
     ## handle union of more than two windows
@@ -302,35 +311,32 @@ intersect.owin <- function(..., fatal=TRUE, p) {
     ## determine a common set of parameters for polyclip
     p <- commonPolyclipArgs(A, B, do.call(boundingbox, windows), p=p)
     ## absorb all windows into B
-    for(i in seq_along(windows))
+    for(i in seq_along(windows)) {
       B <- do.call(intersect.owin,
-                   append(list(B, windows[[i]], p=p),
-                            rasterinfo))
+                   append(list(B, windows[[i]], p=p, fatal=fatal),
+                          rasterinfo))
+      if(is.empty(B)) return(B)
+    }
   }
 
-  ## There are now only two windows
-  if(is.empty(A)) return(A)
-  if(is.empty(B)) return(B)
-
+  ## There are now only two windows, which are not empty.
   if(identical(A, B))
     return(A)
 
   # check units
   if(!compatible(unitname(A), unitname(B)))
     warning("The two windows have incompatible units of length")
+  uname <- harmonise(unitname(A), unitname(B), single=TRUE)
 
   # determine intersection of x and y ranges
   xr <- intersect.ranges(A$xrange, B$xrange, fatal=fatal)
   yr <- intersect.ranges(A$yrange, B$yrange, fatal=fatal)
   if(!fatal && (is.null(xr) || is.null(yr)))
-    return(NULL)
-  C <- owin(xr, yr, unitname=unitname(A))
+    return(emptywindow(A))
+  #' non-empty intersection of Frames
+  C <- owin(xr, yr, unitname=uname)
 
-  if(is.empty(A) || is.empty(B))
-    return(emptywindow(C))
-           
   # Determine type of intersection
-  
   Arect <- is.rectangle(A)
   Brect <- is.rectangle(B)
 #  Apoly <- is.polygonal(A)
@@ -350,13 +356,15 @@ intersect.owin <- function(..., fatal=TRUE, p) {
                   append(list(a, b, "intersection",
                               fillA="nonzero", fillB="nonzero"),
                          p))
-    if(length(ab)==0)
+    if(length(ab)==0) {
+      if(fatal) stop("Intersection is empty", call.=FALSE)
       return(emptywindow(C))
+    }
     # ensure correct polarity
     totarea <- sum(unlist(lapply(ab, Area.xypolygon)))
     if(totarea < 0)
       ab <- lapply(ab, reverse.xypolygon)
-    AB <- owin(poly=ab, check=FALSE, unitname=unitname(A))
+    AB <- owin(poly=ab, check=FALSE, unitname=uname)
     AB <- rescue.rectangle(AB)
     return(AB)
   }
@@ -373,15 +381,29 @@ intersect.owin <- function(..., fatal=TRUE, p) {
   if(Bmask)
     B <- trim.mask(B, C)
 
+  #' trap empty windows
+  if(is.empty(A)) {
+    if(fatal) stop("Intersection is empty", call.=FALSE)
+    return(A)
+  }
+  if(is.empty(B)) {
+    if(fatal) stop("Intersection is empty", call.=FALSE)
+    return(B)
+  }
+  
   # Did the user specify the pixel raster?
   if(length(rasterinfo) > 0) {
     # convert to masks with specified parameters, and intersect
     if(Amask) {
       A <- do.call(as.mask, append(list(A), rasterinfo))
-      return(restrict.mask(A, B))
+      AB <- restrict.mask(A, B)
+      if(fatal && is.empty(AB)) stop("Intersection is empty", call.=FALSE)
+      return(AB)
     } else {
       B <- do.call(as.mask, append(list(B), rasterinfo))
-      return(restrict.mask(B, A))
+      BA <- restrict.mask(B,A)
+      if(fatal && is.empty(BA)) stop("Intersection is empty", call.=FALSE)
+      return(BA)
     }
   } 
   
@@ -392,29 +414,39 @@ intersect.owin <- function(..., fatal=TRUE, p) {
       return(A)
 
   # One mask and one polygon?
-  if(Amask && !Bmask) 
-    return(restrict.mask(A, B))
-  if(!Amask && Bmask)
-    return(restrict.mask(B, A))
+  if(Amask && !Bmask) {
+    AB <- restrict.mask(A, B)
+    if(fatal && is.empty(AB)) stop("Intersection is empty", call.=FALSE)
+    return(AB)
+  }
+  if(!Amask && Bmask) {
+    BA <- restrict.mask(B, A)
+    if(fatal && is.empty(BA)) stop("Intersection is empty", call.=FALSE)
+    return(BA)
+  }
 
   # Two existing masks?
   if(Amask && Bmask) {
     # choose the finer one
-    if(A$xstep <= B$xstep)
-      return(restrict.mask(A, B))
-    else
-      return(restrict.mask(B, A))
+    AB <- if(A$xstep <= B$xstep) restrict.mask(A, B) else restrict.mask(B, A)
+    if(fatal && is.empty(AB)) stop("Intersection is empty", call.=FALSE)
+    return(AB)
   }
 
-  # No existing masks. No clipping applied so far.
-  # Convert one window to a mask with default pixel raster, and intersect.
-  if(Arect) {
-    A <- as.mask(A)
-    return(restrict.mask(A, B))
-  } else {
-    B <- as.mask(B)
-    return(restrict.mask(B, A))
-  }
+  stop("Internal error: never reached")
+#  # No existing masks. No clipping applied so far.
+#  # Convert one window to a mask with default pixel raster, and intersect.
+#  if(Arect) {
+#    A <- as.mask(A)
+#    AB <- restrict.mask(A, B)
+#    if(fatal && is.empty(AB)) stop("Intersection is empty", call.=FALSE)
+#    return(AB)
+#  } else {
+#    B <- as.mask(B)
+#    BA <- restrict.mask(B, A)
+#    if(fatal && is.empty(BA)) stop("Intersection is empty", call.=FALSE)
+#    return(BA)
+#  }
 }
 
 
@@ -437,7 +469,7 @@ union.owin <- function(..., p) {
     }
   }
   ## look for window arguments
-  isowin <- sapply(argh, is.owin)
+  isowin <- as.logical(sapply(argh, is.owin))
   if(any(!isowin))
     warning("Some arguments were not windows")
   argh <- argh[isowin]
@@ -491,6 +523,7 @@ union.owin <- function(..., p) {
   ## check units
   if(!compatible(unitname(A), unitname(B)))
     warning("The two windows have incompatible units of length")
+  uname <- harmonise(unitname(A), unitname(B), single=TRUE)
   
   ## Determine type of intersection
   
@@ -505,7 +538,7 @@ union.owin <- function(..., p) {
   
   C <- owin(range(A$xrange, B$xrange),
             range(A$yrange, B$yrange),
-            unitname=unitname(A))
+            unitname=uname)
 
   if(!Amask && !Bmask) {
     ####### Result is polygonal (or rectangular) ############
@@ -521,7 +554,7 @@ union.owin <- function(..., p) {
     totarea <- sum(unlist(lapply(ab, Area.xypolygon)))
     if(totarea < 0)
       ab <- lapply(ab, reverse.xypolygon)
-    AB <- owin(poly=ab, check=FALSE, unitname=unitname(A))
+    AB <- owin(poly=ab, check=FALSE, unitname=uname)
     AB <- rescue.rectangle(AB)
     return(AB)
   }
@@ -574,6 +607,7 @@ setminus.owin <- function(A, B, ..., p) {
   ## check units
   if(!compatible(unitname(A), unitname(B)))
     warning("The two windows have incompatible units of length")
+  uname <- harmonise(unitname(A), unitname(B), single=TRUE)
   
   ## Determine type of arguments
   
@@ -609,7 +643,7 @@ setminus.owin <- function(A, B, ..., p) {
     totarea <- sum(unlist(lapply(ab, Area.xypolygon)))
     if(totarea < 0)
       ab <- lapply(ab, reverse.xypolygon)
-    AB <- owin(poly=ab, check=FALSE, unitname=unitname(A))
+    AB <- owin(poly=ab, check=FALSE, unitname=uname)
     AB <- rescue.rectangle(AB)
     return(AB)
   }
@@ -919,6 +953,7 @@ incircle <- function(W) {
   v <- D$v
   ok <- !is.na(v)
   Dvalues <- as.vector(v[ok])
+  if(length(Dvalues) == 0) return(NULL)
   Dmax <- max(Dvalues)
   # find location of maximum
   locn <- which.max(Dvalues)
@@ -948,6 +983,7 @@ inpoint <- function(W) {
   }
   W <- as.mask(W)
   Mm <- W$m
+  if(!any(Mm)) return(NULL)
   Mrow <- as.vector(row(Mm)[Mm])
   Mcol <- as.vector(col(Mm)[Mm])
   selectmiddle <- function(x) { x[ceiling(length(x)/2)] }
